@@ -26,6 +26,11 @@
 #include "refaccess.h"
 #include "settings.h"
 
+#ifdef HAVE_LIBGSL  
+#include <gsl/gsl_sf.h>
+#include <gsl/gsl_errno.h>
+#endif
+  
 using namespace types;
 using namespace camp;
 
@@ -46,7 +51,7 @@ void gen_base_venv(venv &ve);
 
 void addType(tenv &te, const char *name, ty *t)
 {
-  te.enter(symbol::trans(name), new tyEntry(t,0));
+  te.enter(symbol::trans(name), new tyEntry(t,0,0));
 }
 
 // The base environments for built-in types and functions
@@ -89,7 +94,9 @@ void addFunc(venv &ve, access *a, ty *result, symbol *id,
   if (f7.t) fun->add(f7);
   if (f8.t) fun->add(f8);
 
-  varEntry *ent = new varEntry(fun, a);
+  // NOTE: If the function is a field, we should encode the defining record in
+  // the entry
+  varEntry *ent = new varEntry(fun, a, 0);
   
   ve.enter(id, ent);
 }
@@ -129,7 +136,7 @@ void addRestFunc(venv &ve, bltin f, ty *result, const char *name, formal frest,
 
   if (frest.t) fun->addRest(frest);
 
-  varEntry *ent = new varEntry(fun, a);
+  varEntry *ent = new varEntry(fun, a, 0);
 
   ve.enter(symbol::trans(name), ent);
 }
@@ -143,18 +150,158 @@ template<double (*fcn)(double)>
 void addRealFunc(venv &ve, const char* name)
 {
   addFunc(ve, realReal<fcn>, primReal(), name, formal(primReal(),"x"));
-  addFunc(ve, realArrayFunc<fcn>, realArray(), name, 
-	  formal(realArray(),"a"));
+  addFunc(ve, realArrayFunc<fcn>, realArray(), name, formal(realArray(),"a"));
 }
 
 #define addRealFunc(fcn) addRealFunc<fcn>(ve, #fcn);
-
+  
 void addRealFunc2(venv &ve, bltin fcn, const char *name)
 {
   addFunc(ve,fcn,primReal(),name,formal(primReal(),"a"),
 	  formal(primReal(),"b"));
 }
 
+#ifdef HAVE_LIBGSL  
+bool GSLerror=false;
+  
+types::dummyRecord *GSLModule;
+
+types::record *getGSLModule()
+{
+  return GSLModule;
+}
+
+inline void checkGSLerror()
+{
+  if(GSLerror) {
+    GSLerror=false;
+    throw handled_error();
+  }
+}
+  
+template <double (*func)(double)>
+void realRealGSL(vm::stack *s) 
+{
+  double x=vm::pop<double>(s);
+  s->push(func(x));
+  checkGSLerror();
+}
+
+template <double (*func)(double, gsl_mode_t)>
+void realRealDOUBLE(vm::stack *s) 
+{
+  double x=vm::pop<double>(s);
+  s->push(func(x,GSL_PREC_DOUBLE));
+  checkGSLerror();
+}
+
+template <double (*func)(double, double, gsl_mode_t)>
+void realRealRealDOUBLE(vm::stack *s) 
+{
+  double y=vm::pop<double>(s);
+  double x=vm::pop<double>(s);
+  s->push(func(x,y,GSL_PREC_DOUBLE));
+  checkGSLerror();
+}
+
+template <double (*func)(unsigned int)>
+void realIntGSL(vm::stack *s) 
+{
+  int n=vm::pop<int>(s);
+  if(n < 0) n=0;
+  s->push(func(n));
+  checkGSLerror();
+}
+
+template <double (*func)(int, double)>
+void realIntRealGSL(vm::stack *s) 
+{
+  double x=vm::pop<double>(s);
+  int n=vm::pop<int>(s);
+  s->push(func(n,x));
+  checkGSLerror();
+}
+
+template <double (*func)(double, double)>
+void realRealRealGSL(vm::stack *s) 
+{
+  double x=vm::pop<double>(s);
+  double n=vm::pop<double>(s);
+  s->push(func(n,x));
+  checkGSLerror();
+}
+
+template <double (*func)(double, unsigned int)>
+void realRealIntGSL(vm::stack *s) 
+{
+  int n=vm::pop<int>(s);
+  double x=vm::pop<double>(s);
+  if(n < 0) n=0;
+  s->push(func(x,n));
+  checkGSLerror();
+}
+
+// Add a GSL special function from the GNU GSL library
+template<double (*fcn)(double)>
+void addGSLRealFunc(const char* name)
+{
+  addFunc(GSLModule->e.ve, realRealGSL<fcn>, primReal(), name,
+	  formal(primReal(),"x"));
+}
+
+// Add a GSL_PREC_DOUBLE GSL special function.
+template<double (*fcn)(double, gsl_mode_t)>
+void addGSLDOUBLEFunc(const char* name)
+{
+  addFunc(GSLModule->e.ve, realRealDOUBLE<fcn>, primReal(), name,
+	  formal(primReal(),"x"));
+}
+
+template<double (*fcn)(double, double, gsl_mode_t)>
+void addGSLDOUBLE2Func(const char* name)
+{
+  addFunc(GSLModule->e.ve, realRealRealDOUBLE<fcn>, primReal(), name, 
+	  formal(primReal(),"phi"), formal(primReal(),"k"));
+}
+
+template<double (*fcn)(unsigned int)>
+void addGSLIntFunc(const char* name)
+{
+  addFunc(GSLModule->e.ve, realIntGSL<fcn>, primReal(), name,
+	  formal(primInt(),"s"));
+}
+
+template<double (*fcn)(int, double)>
+void addGSLIntRealFunc(const char* name, const char *arg1="n")
+{
+  addFunc(GSLModule->e.ve, realIntRealGSL<fcn>, primReal(), name,
+	  formal(primInt(),arg1), formal(primReal(),"x"));
+}
+
+template<double (*fcn)(double, double)>
+void addGSLRealRealFunc(const char* name)
+{
+  addFunc(GSLModule->e.ve, realRealRealGSL<fcn>, primReal(), name,
+	  formal(primReal(),"nu"), formal(primReal(),"x"));
+}
+
+template<double (*fcn)(double, unsigned int)>
+void addGSLRealIntFunc(const char* name)
+{
+  addFunc(GSLModule->e.ve, realRealIntGSL<fcn>, primReal(), name, 
+	  formal(primReal(),"nu"), formal(primInt(),"s"));
+}
+
+// Handle GSL errors gracefully.
+void GSLerrorhandler(const char *reason, const char *, int, int) 
+{
+  if(!GSLerror) {
+    vm::errornothrow(reason);
+    GSLerror=true;
+  }
+}
+#endif
+  
 void addInitializer(venv &ve, ty *t, access *a)
 {
   addFunc(ve, a, t, symbol::initsym);
@@ -188,11 +335,12 @@ void addCast(venv &ve, ty *target, ty *source, bltin f) {
 }
 
 template<class T>
-void addConstant(venv &ve, T value, ty *t, const char *name) {
+void addConstant(venv &ve, T value, ty *t, const char *name,
+                 record *module=settings::getSettingsModule()) {
   item* ref=new item;
   *ref=value;
   access *a = new itemRefAccess(ref);
-  varEntry *ent = new varEntry(t, a, READONLY, settings::getSettingsModule());
+  varEntry *ent = new varEntry(t, a, RESTRICTED, module, 0);
   ve.enter(symbol::trans(name), ent);
 }
 
@@ -380,6 +528,57 @@ void addOps(venv &ve, ty *t1, ty *t2, ty *t3, ty *t4, bool divide=true)
   addOps<T,power>(ve,t1,"^",t2);
 }
 
+
+// Adds standard functions for a newly added array type.
+void addArrayOps(venv &ve, types::array *t)
+{
+  ty *ct = t->celltype;
+  
+  addFunc(ve, run::arrayAlias,
+          primBoolean(), "alias", formal(t, "a"), formal(t, "b"));
+
+  switch (t->depth()) {
+  case 1:
+    addFunc(ve, run::arrayCopy, t, "copy", formal(t, "a"));
+    addFunc(ve, run::arrayConcat, t, "concat", formal(t, "a"), formal(t, "b"));
+    addFunc(ve, run::arraySequence,
+            t, "sequence", formal(new function(ct, primInt()), "f"),
+                           formal(primInt(), "n"));
+    addFunc(ve, run::arrayFunction,
+            t, "map", formal(new function(ct, ct), "f"), formal(t, "a"));
+    break;
+  case 2:
+    addFunc(ve, run::array2Copy, t, "copy", formal(t, "a"));
+    addFunc(ve, run::array2Transpose, t, "transpose", formal(t, "a"));
+    break;
+  case 3:
+    addFunc(ve, run::array3Copy, t, "copy", formal(t, "a"));
+    addFunc(ve, run::array3Transpose, t, "transpose", formal(t, "a"),
+	    formal(intArray(),"perm"));
+    break;
+  default:
+    break;
+  }
+}
+
+void addRecordOps(venv &ve, record *r)
+{
+  addFunc(ve, run::boolMemEq, primBoolean(), "alias", types::formal(r, "a"),
+          types::formal(r, "b"));
+  addFunc(ve, run::boolMemEq, primBoolean(), "==", types::formal(r, "a"),
+          types::formal(r, "b"));
+  addFunc(ve, run::boolMemNeq, primBoolean(), "!=", types::formal(r, "a"),
+          types::formal(r, "b"));
+}
+
+void addFunctionOps(venv &ve, function *f)
+{
+  addFunc(ve, run::boolFuncEq, primBoolean(), "==", types::formal(f, "a"),
+          types::formal(f, "b"));
+  addFunc(ve, run::boolFuncNeq, primBoolean(), "!=", types::formal(f, "a"),
+          types::formal(f, "b"));
+}
+
 void addOperators(venv &ve) 
 {
   addSimpleOperator(ve,binaryOp<string,plus>,primString(),"+");
@@ -416,6 +615,14 @@ void addOperators(venv &ve)
   addOps<double,mod>(ve,primReal(),"%",realArray());
 }
 
+dummyRecord *createDummyRecord(venv &ve, const char *name)
+{
+  dummyRecord *r=new dummyRecord(name);
+  addConstant(ve, new vm::frame(0), r, name);
+  addRecordOps(ve, r);
+  return r;
+}
+
 double identity(double x) {return x;}
 double pow10(double x) {return pow(10.0,x);}
 
@@ -450,6 +657,44 @@ void base_venv(venv &ve)
   addRealFunc(pow10);
   addRealFunc(identity);
   
+#ifdef HAVE_LIBGSL  
+  GSLModule=new types::dummyRecord(symbol::trans("gsl"));
+  gsl_set_error_handler(GSLerrorhandler);
+  
+  addGSLDOUBLEFunc<gsl_sf_airy_Ai>("Ai");
+  addGSLDOUBLEFunc<gsl_sf_airy_Bi>("Bi");
+  addGSLDOUBLEFunc<gsl_sf_airy_Ai_deriv>("Ai_deriv");
+  addGSLDOUBLEFunc<gsl_sf_airy_Bi_deriv>("Bi_deriv");
+  addGSLIntFunc<gsl_sf_airy_zero_Ai>("zero_Ai");
+  addGSLIntFunc<gsl_sf_airy_zero_Bi>("zero_Bi");
+  addGSLIntFunc<gsl_sf_airy_zero_Ai_deriv>("zero_Ai_deriv");
+  addGSLIntFunc<gsl_sf_airy_zero_Bi_deriv>("zero_Bi_deriv");
+  
+  addGSLIntRealFunc<gsl_sf_bessel_In>("I");
+  addGSLIntRealFunc<gsl_sf_bessel_Kn>("K");
+  addGSLIntRealFunc<gsl_sf_bessel_jl>("j","l");
+  addGSLIntRealFunc<gsl_sf_bessel_yl>("y","l");
+  addGSLIntRealFunc<gsl_sf_bessel_il_scaled>("i_scaled","l");
+  addGSLIntRealFunc<gsl_sf_bessel_kl_scaled>("k_scaled","l");
+  addGSLRealRealFunc<gsl_sf_bessel_Jnu>("J");
+  addGSLRealRealFunc<gsl_sf_bessel_Ynu>("Y");
+  addGSLRealRealFunc<gsl_sf_bessel_Inu>("I");
+  addGSLRealRealFunc<gsl_sf_bessel_Knu>("K");
+  addGSLRealIntFunc<gsl_sf_bessel_zero_Jnu>("zero_J");
+  
+  addGSLDOUBLE2Func<gsl_sf_ellint_E>("F");
+  addGSLDOUBLE2Func<gsl_sf_ellint_E>("E");
+  addGSLDOUBLE2Func<gsl_sf_ellint_E>("P");
+  
+  addGSLRealFunc<gsl_sf_expint_Ei>("Ei");
+  addGSLRealFunc<gsl_sf_Si>("Si");
+  addGSLRealFunc<gsl_sf_Ci>("Ci");
+  
+  addGSLIntRealFunc<gsl_sf_legendre_Pl>("Pl","l");
+  
+  addGSLRealFunc<gsl_sf_zeta>("zeta");
+#endif
+  
   addFunc(ve,writestring,primVoid(),"write",
 	  formal(primFile(),"file",true),
 	  formal(primString(),"s"),
@@ -476,6 +721,7 @@ void base_venv(venv &ve)
   addConstant<double>(ve, DBL_MAX, primReal(), "realMax");
   addConstant<double>(ve, DBL_MIN, primReal(), "realMin");
   addConstant<double>(ve, DBL_EPSILON, primReal(), "realEpsilon");
+  addConstant<int>(ve, DBL_DIG, primInt(), "realDigits");
   addConstant<int>(ve, RAND_MAX, primInt(), "randMax");
   addConstant<double>(ve, PI, primReal(), "pi");
 
