@@ -22,9 +22,9 @@ namespace absyntax {
 using namespace trans;
 using namespace types;
   
-trans::tyEntry *ty::transAsTyEntry(coenv &e)
+trans::tyEntry *ty::transAsTyEntry(coenv &e, record *where)
 {
-  return new trans::tyEntry(trans(e, false), 0);
+  return new trans::tyEntry(trans(e, false), 0, where);
 }
 
 
@@ -40,12 +40,35 @@ types::ty *nameTy::trans(coenv &e, bool tacit)
   return id->typeTrans(e, tacit);
 }
 
-trans::tyEntry *nameTy::transAsTyEntry(coenv &e)
+trans::tyEntry *nameTy::transAsTyEntry(coenv &e, record *)
 {
   return id->tyEntryTrans(e);
 }
 
 
+void dimensions::prettyprint(ostream &out, int indent)
+{
+  prettyindent(out, indent);
+  out << "dimensions (" << depth << ")\n";
+}
+
+types::array *dimensions::truetype(types::ty *base)
+{
+  if (base->kind == ty_void) {
+    em->compiler(getPos());
+    *em << "cannot declare array of type void";
+  }
+
+  assert(depth >= 1);
+  size_t d=depth;
+  types::array *a=new types::array(base); d--;
+  for (; d > 0; d--) {
+    a = new types::array(a);
+  }
+  return a;
+}
+
+
 void arrayTy::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "arrayTy",indent);
@@ -54,85 +77,20 @@ void arrayTy::prettyprint(ostream &out, int indent)
   dims->prettyprint(out, indent+1);
 }
 
-function *arrayTy::opType(types::ty* t)
-{
-  return new function(primBoolean(),types::formal(t,"a"),types::formal(t,"b"));
-}
-
-function *arrayTy::arrayType(types::ty* t)
-{
-  return new function(t,types::formal(t,"a"));
-}
-
-function *arrayTy::array2Type(types::ty* t)
-{
-  return new function(t,types::formal(t,"a"),types::formal(t,"b"));
-}
-
-function *arrayTy::cellIntType(types::ty* t)
-{
-  return new function(t,primInt());
-}
-  
-function *arrayTy::sequenceType(types::ty* t, types::ty *ct)
-{
-  return new function(t,types::formal(cellIntType(ct),"f"),
-		      types::formal(primInt(),"n"));
-}
-
-function *arrayTy::cellTypeType(types::ty* t)
-{
-  return new function(t,t);
-}
-  
-function *arrayTy::mapType(types::ty* t, types::ty *ct)
-{
-  return new function(t,types::formal(cellTypeType(ct),"f"),
-		      types::formal(t,"a"));
-}
-
-void arrayTy::addOps(coenv &e, types::ty* t, types::ty *ct)
-{
-  function *ft = opType(t);
-  function *ftarray = arrayType(t);
-  function *ftarray2 = array2Type(t);
-  function *ftsequence = sequenceType(t,ct);
-  function *ftmap = mapType(t,ct);
-  
-  e.e.addVar(symbol::trans("alias"),
-      new varEntry(ft,new bltinAccess(run::arrayAlias)));
-
-  if(dims->size() == 1) {
-    e.e.addVar(symbol::trans("copy"),
-	       new varEntry(ftarray,new bltinAccess(run::arrayCopy)));
-    e.e.addVar(symbol::trans("concat"),
-	       new varEntry(ftarray2,new bltinAccess(run::arrayConcat)));
-    e.e.addVar(symbol::trans("sequence"),
-	       new varEntry(ftsequence,new bltinAccess(run::arraySequence)));
-    e.e.addVar(symbol::trans("map"),
-	       new varEntry(ftmap,new bltinAccess(run::arrayFunction)));
-  }
-  if(dims->size() == 2) {
-    e.e.addVar(symbol::trans("copy"),
-	       new varEntry(ftarray,new bltinAccess(run::array2Copy)));
-    e.e.addVar(symbol::trans("transpose"),
-	       new varEntry(ftarray,new bltinAccess(run::array2Transpose)));
-  }
-}
-
 types::ty *arrayTy::trans(coenv &e, bool tacit)
 {
   types::ty *ct = cell->trans(e, tacit);
   assert(ct);
 
-  types::ty *t = dims->truetype(ct);
+  types::array *t = dims->truetype(ct);
   assert(t);
   
-  addOps(e,t,ct);
+  e.e.addArrayOps(t);
   
   return t;
 }
 
+
 vm::lambda *runnable::transAsCodelet(coenv &e)
 {
   coder c=e.c.newCodelet();
@@ -178,15 +136,27 @@ void block::transAsRecordBody(coenv &e, record *r)
   e.c.closeRecord();
 }
 
-void block::transAsFile(coenv &e, record *r)
+record *block::transAsFile(genv& ge, symbol *id)
 {
-  if (settings::getSetting<bool>("autoplain")) {
-    autoplainRunnable()->transAsField(e, r);
-  }
+  // Create the new module.
+  record *r = new record(id, new frame(0,0));
 
-  transAsRecordBody(e, r);
-}
+  // Create coder and environment to translate the module.
+  // File-level modules have dynamic fields by default.
+  coder c(r, 0);
+  env e(ge);
+  coenv ce(c, e);
+
+  // Translate the abstract syntax.
+  if (settings::getSetting<bool>("autoplain")) {
+    autoplainRunnable()->transAsField(ce, r);
+  }
+  transAsRecordBody(ce, r);
+  em->sync();
   
+  return r;
+}
+
 bool block::returns() {
   // Search for a returning runnable, starting at the end for efficiency.
   for (list<runnable *>::reverse_iterator p=stms.rbegin();
@@ -267,7 +237,7 @@ permission modifierList::getPermission()
     *em << "too many modifiers";
   }
 
-  return perms.empty() ? READONLY : perms.front();
+  return perms.empty() ? DEFAULT_PERM : perms.front();
 }
 
 
@@ -281,12 +251,17 @@ void modifiedRunnable::prettyprint(ostream &out, int indent)
 
 void modifiedRunnable::transAsField(coenv &e, record *r)
 {
-  if (mods->staticSet())
+  if (mods->staticSet()) {
+    if (e.c.isTopLevel()) {
+      em->warning(getPos());
+      *em << "static modifier is meaningless at top level";
+    }
     e.c.pushModifier(mods->getModifier());
+  }
 
   permission p = mods->getPermission();
 #if 0 // This is innocuous 
-  if (p != READONLY && (!r || !body->allowPermissions())) {
+  if (p != DEFAULT_PERM && (!r || !body->allowPermissions())) {
     em->warning(pos);
     *em << "permission modifier is meaningless";
   }
@@ -315,12 +290,22 @@ types::ty *decidstart::getType(types::ty *base, coenv &, bool)
   return dims ? dims->truetype(base) : base;
 }
 
-trans::tyEntry *decidstart::getTyEntry(trans::tyEntry *base, coenv &e)
+trans::tyEntry *decidstart::getTyEntry(trans::tyEntry *base, coenv &e,
+                                       record *where)
 {
-  return dims ? new trans::tyEntry(getType(base->t,e,false), 0) :
+  return dims ? new trans::tyEntry(getType(base->t,e,false), 0, where) :
                 base;
 }
 
+void decidstart::addOps(types::ty *base, coenv &e, record *r)
+{
+  if (dims) {
+    e.e.addArrayOps(dims->truetype(base));
+    if (r)
+      r->e.addArrayOps(dims->truetype(base));
+  }
+}
+
 
 void fundecidstart::prettyprint(ostream &out, int indent)
 {
@@ -346,11 +331,25 @@ types::ty *fundecidstart::getType(types::ty *base, coenv &e, bool tacit)
   }
 }
 
-trans::tyEntry *fundecidstart::getTyEntry(trans::tyEntry *base, coenv &e)
+trans::tyEntry *fundecidstart::getTyEntry(trans::tyEntry *base, coenv &e,
+                                          record *where)
 {
-  return new trans::tyEntry(getType(base->t,e,false), 0);
+  return new trans::tyEntry(getType(base->t,e,false), 0, where);
 }
 
+void fundecidstart::addOps(types::ty *base, coenv &e, record *r)
+{
+  decidstart::addOps(base, e, r);
+
+  types::function *ft=dynamic_cast<types::function *>(getType(base, e, true));
+  assert(ft);
+
+  e.e.addFunctionOps(ft);
+  if (r)
+    r->e.addFunctionOps(ft);
+}
+
+
 void decid::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "decid",indent);
@@ -361,13 +360,18 @@ void decid::prettyprint(ostream &out, int indent)
 }
 
 
-varEntry *makeVarEntry(coenv &e, record *r, types::ty *t)
+varEntry *makeVarEntryWhere(coenv &e, record *r, types::ty *t,
+                                record *where)
 {
   access *a = r ? r->allocField(e.c.isStatic()) :
                   e.c.allocLocal();
 
-  return r ? new varEntry(t, a, e.c.getPermission(), r) :
-             new varEntry(t, a);
+  return r ? new varEntry(t, a, e.c.getPermission(), r, where) :
+             new varEntry(t, a, where);
+}
+
+varEntry *makeVarEntry(coenv &e, record *r, types::ty *t) {
+  return makeVarEntryWhere(e, r, t, r);
 }
 
 void addVar(coenv &e, record *r, varEntry *v, symbol *id)
@@ -402,6 +406,18 @@ void createVar(position pos, coenv &e, record *r,
   initializeVar(pos, e, r, v, t, init);
 }
 
+void addTypeWithPermission(coenv &e, record *r, tyEntry *base, symbol *id)
+{
+  // Only bother encoding permissions for private types.
+  tyEntry *ent = (r && e.c.getPermission()==PRIVATE) ?
+                     new trans::tyEntry(base, PRIVATE, r) :
+                     base;
+
+  if (r)
+    r->e.addType(id, ent);
+  e.e.addType(id, ent);
+}
+
 void createVarOutOfOrder(position pos, coenv &e, record *r,
                          symbol *id, types::ty *t, varinit *init)
 {
@@ -410,22 +426,23 @@ void createVarOutOfOrder(position pos, coenv &e, record *r,
   addVar(e, r, v, id);
 }
 
-
 void decid::transAsField(coenv &e, record *r, types::ty *base)
 {
   types::ty *t = start->getType(base, e);
   assert(t);
   if (t->kind == ty_void) {
     em->compiler(getPos());
-    *em << "can't declare variable of type void";
+    *em << "cannot declare variable of type void";
   }
 
+  start->addOps(base, e, r);
+
   createVarOutOfOrder(getPos(), e, r, start->getName(), t, init);
 }
 
 void decid::transAsTypedefField(coenv &e, trans::tyEntry *base, record *r)
 {
-  trans::tyEntry *ent = start->getTyEntry(base, e);
+  trans::tyEntry *ent = start->getTyEntry(base, e, r);
   assert(ent && ent->t);
 
   if (init) {
@@ -433,10 +450,9 @@ void decid::transAsTypedefField(coenv &e, trans::tyEntry *base, record *r)
     *em << "type definition cannot have initializer";
   }
    
-  // Add to type to record and environment.
-  if (r)
-    r->e.addType(start->getName(), ent);
-  e.e.addType(start->getName(), ent);
+  start->addOps(base->t, e, r);
+
+  addTypeWithPermission(e, r, ent, start->getName());
 }
 
 
@@ -471,7 +487,7 @@ void vardec::prettyprint(ostream &out, int indent)
 
 void vardec::transAsTypedefField(coenv &e, record *r)
 {
-  decs->transAsTypedefField(e, base->transAsTyEntry(e), r);
+  decs->transAsTypedefField(e, base->transAsTyEntry(e, r), r);
 }
 
 // Helper class for imports.  This essentially evaluates to the run::loadModule
@@ -515,7 +531,7 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
   record *imp=e.e.getModule(id, (mem::string)*id);
   if (!imp) {
     em->error(pos);
-    *em << "could not load module of name '" << *id << "'";
+    *em << "could not load module '" << *id << "'";
     em->sync();
     return 0;
   }
@@ -525,7 +541,9 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
     callExp init(pos, new loadModuleExp(pos, imp),
                       new stringExp(pos, *id));
 
-    varEntry *v=makeVarEntry(e, r, imp);
+    // The varEntry should have whereDefined()==0 as it is not defined inside
+    // the record r.
+    varEntry *v=makeVarEntryWhere(e, r, imp, 0);
     initializeVar(pos, e, r, v, imp, &init);
     return v;
   }
@@ -700,33 +718,15 @@ void recorddec::prettyprint(ostream &out, int indent)
   body->prettyprint(out, indent+1);
 }
 
-function *recorddec::opType(record *r)
-{
-  return new function(primBoolean(),types::formal(r,"a"),
-		      types::formal(r,"b"));
-}
-
-void recorddec::addOps(coenv &e, record *parent, record *r)
-{
-  function *ft = opType(r);
-  varEntry *ve=new varEntry(ft, new bltinAccess(run::boolMemEq));
-  addVar(e,parent,ve,symbol::trans("alias"));
-  addVar(e,parent,ve,symbol::trans("=="));
-  addVar(e,parent,new varEntry(ft, new bltinAccess(run::boolMemNeq)),
-	 symbol::trans("!="));
-}
-
 void recorddec::transAsField(coenv &e, record *parent)
 {
   record *r = parent ? parent->newRecord(id, e.c.isStatic()) :
                        e.c.newRecord(id);
                      
-  tyEntry *ent = new trans::tyEntry(r,0);
-
+  addTypeWithPermission(e, parent, new trans::tyEntry(r,0,parent), id);
+  e.e.addRecordOps(r);
   if (parent)
-    parent->e.addType(id, ent);
-  e.e.addType(id, ent);
-  addOps(e,parent,r);
+    parent->e.addRecordOps(r);
 
   // Start translating the initializer.
   coder c=e.c.newRecordInit(r);
@@ -736,7 +736,8 @@ void recorddec::transAsField(coenv &e, record *parent)
 }  
 
 runnable *autoplainRunnable() {
-  // Private import plain;
+  // Abstract syntax for the code:
+  //   private import plain;
   position pos=position();
   static importdec ap(pos, new idpair(pos, symbol::trans("plain")));
   static modifiedRunnable mr(pos, trans::PRIVATE, &ap);
