@@ -18,6 +18,7 @@
 
 namespace trans {
 class coenv;
+class genv;
 class protoenv;
 class varEntry;
 class access;
@@ -36,6 +37,7 @@ class lambda;
 namespace absyntax {
 
 using mem::list;
+using trans::genv;
 using trans::coenv;
 using trans::protoenv;
 using trans::varEntry;
@@ -54,7 +56,7 @@ public:
   // needed to silence errors in this case.
   virtual types::ty *trans(coenv &e, bool tacit = false) = 0;
 
-  virtual trans::tyEntry *transAsTyEntry(coenv &e);
+  virtual trans::tyEntry *transAsTyEntry(coenv &e, record *where);
 };
 
 class nameTy : public ty {
@@ -70,7 +72,7 @@ public:
   void prettyprint(ostream &out, int indent);
 
   types::ty *trans(coenv &e, bool tacit = false);
-  trans::tyEntry *transAsTyEntry(coenv &e);
+  trans::tyEntry *transAsTyEntry(coenv &e, record *where);
 };
 
 class dimensions : public absyn {
@@ -88,7 +90,7 @@ public:
     return depth;
   }
 
-  types::ty *truetype(types::ty *base);
+  types::array *truetype(types::ty *base);
 };
 
 class arrayTy : public ty {
@@ -104,16 +106,6 @@ public:
 
   void prettyprint(ostream &out, int indent);
 
-  types::function *opType(types::ty* t);
-  types::function *arrayType(types::ty* t);
-  types::function *array2Type(types::ty* t);
-  types::function *cellIntType(types::ty* t);
-  types::function *sequenceType(types::ty* t, types::ty *ct);
-  types::function *cellTypeType(types::ty* t);
-  types::function *mapType(types::ty* t, types::ty *ct);
-  
-  void addOps(coenv &e, types::ty* t, types::ty *ct);
-  
   types::ty *trans(coenv &e, bool tacit = false);
 };
 
@@ -192,7 +184,7 @@ public:
 
   void transAsRecordBody(coenv &e, record *r);
 
-  void transAsFile(coenv &e, record *r);
+  types::record *transAsFile(genv& ge, symbol *id);
 
   // A block is guaranteed to return iff one of the runnables is guaranteed to
   // return.
@@ -282,7 +274,12 @@ public:
   virtual void prettyprint(ostream &out, int indent);
 
   virtual types::ty *getType(types::ty *base, coenv &, bool = false);
-  virtual trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e);
+  virtual trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e,
+                                     record *where);
+
+  // If a new type is formed by adding dimensions (or a function signature)
+  // after the id, this will add the standard functions for that new type.
+  virtual void addOps(types::ty *base, coenv &e, record *r);
 
   virtual symbol *getName()
     { return id; }
@@ -304,7 +301,8 @@ public:
   void prettyprint(ostream &out, int indent);
 
   types::ty *getType(types::ty *base, coenv &e, bool tacit = false);
-  virtual trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e);
+  trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e, record *where);
+  void addOps(types::ty *base, coenv &e, record *r);
 };
 
 class decid : public absyn {
@@ -551,9 +549,6 @@ class recorddec : public dec {
   symbol *id;
   block *body;
 
-  types::function *opType(record *r);
-  void addOps(coenv &e, record *parent, record *r);
-
 public:
   recorddec(position pos, symbol *id, block *body)
     : dec(pos), id(id), body(body) {}
