@@ -41,7 +41,7 @@ bool drawLabel::texbounds(iopipestream& tex, string& s, bool warn)
       tex << "\n";
       tex.wait("\n*","! ");
       return false;
-    } else reportError("Can't read label width");
+    } else reportError("Cannot read label width");
   }
   tex << "\n";
   tex.wait("\n*","! ");
@@ -49,78 +49,76 @@ bool drawLabel::texbounds(iopipestream& tex, string& s, bool warn)
   tex >> texbuf;
   if(texbuf[0] == '>' && texbuf[1] == ' ')
     height=atof(texbuf.c_str()+2)*tex2ps;
-  else reportError("Can't read label height");
+  else reportError("Cannot read label height");
   tex << "\n";
   tex.wait("\n*","! ");
   tex << "\\showthe\\dp\\ASYbox\n";
   tex >> texbuf;
   if(texbuf[0] == '>' && texbuf[1] == ' ')
     depth=atof(texbuf.c_str()+2)*tex2ps;
-  else reportError("Can't read label depth");
+  else reportError("Cannot read label depth");
   tex << "\n";
   tex.wait("\n*","! ");
      
-  width *= fontscale;
-  height *= fontscale;
-  depth *= fontscale;
   return true;
 }   
 
+inline double urand()
+{			  
+  static const double factor=2.0/RAND_MAX;
+  return rand()*factor-1.0;
+}
 
 void drawLabel::bounds(bbox& b, iopipestream& tex, boxvector& labelbounds,
 		       bboxlist&)
 {
-  if(!settings::getSetting<bool>("tex")) {b += position; return;}
-  pair rotation=expi(radians(angle));
-  pen Pentype=pentype;
-  static const double fuzz=1.0+Pentype.size()/24.0;
+  mem::string texengine=settings::getSetting<mem::string>("tex");
+  if(texengine == "none") {b += position; return;}
   
   if(!havebounds) {
     havebounds=true;
-    if(Pentype.size() != lastpen.size() ||
-       Pentype.Lineskip() != lastpen.Lineskip()) {
-      tex <<  "\\fontsize{" << Pentype.size() << "}{" << Pentype.Lineskip()
+    if(pentype.size() != lastpen.size() ||
+       pentype.Lineskip() != lastpen.Lineskip()) {
+      if(settings::latex(texengine)) {
+      tex <<  "\\fontsize{" << pentype.size() << "}{" << pentype.Lineskip()
 	  << "}\\selectfont\n";
       tex.wait("\n*","! ");
     }
     
-    string font=Pentype.Font();
+    mem::string font=pentype.Font();
     if(font != lastpen.Font()) {
-      fontscale=1.0;
-      string scaled=" scaled";
-      size_t p=font.find(scaled);
-      if(p < string::npos) {
-	p += scaled.length();
-	fontscale=atof(font.substr(p,string::npos).c_str())/1000.0;
+	tex <<  font << "\n";
+	tex.wait("\n*","! ");
       }
-      tex <<  font << "\n";
-      tex.wait("\n*","! ");
     }
     
-    lastpen=Pentype;
+    lastpen=pentype;
     
     bool nullsize=size == "";
     if(!texbounds(tex,label,nullsize) && !nullsize)
       texbounds(tex,size,false);
     
-    Align=align/rotation;
+    Align=inverse(T)*align;
     double scale0=max(fabs(Align.getx()),fabs(Align.gety()));
     if(scale0) Align *= 0.5/scale0;
     Align -= pair(0.5,0.5);
-    double Depth=(Pentype.Baseline() == NOBASEALIGN) ? depth : 0.0;
+    double Depth=(pentype.Baseline() == NOBASEALIGN) ? depth : 0.0;
     texAlign=Align;
     if(Depth > 0) texAlign += pair(0.0,Depth/(height+Depth));
     Align.scale(width,height+Depth);
     Align += pair(0.0,Depth-depth);
-    Align *= rotation;
+    Align=T*Align;
   }
 
   // alignment point
   pair p=position+Align;
-  pair A=p+pair(-fuzz,-fuzz)*rotation;
-  pair B=p+pair(-fuzz,height+depth+fuzz)*rotation;
-  pair C=p+pair(width+fuzz,height+depth+fuzz)*rotation;
-  pair D=p+pair(width+fuzz,-fuzz)*rotation;
+  const double vertical=height+depth;
+  static const double epsilon=0.09;
+  const double fuzz=vertical*epsilon;
+  pair A=p+T*pair(-fuzz,-fuzz);
+  pair B=p+T*pair(-fuzz,vertical+fuzz);
+  pair C=p+T*pair(width+fuzz,vertical+fuzz);
+  pair D=p+T*pair(width+fuzz,-fuzz);
   
   if(pentype.Overwrite() != ALLOW && label != "") {
     size_t n=labelbounds.size();
@@ -139,7 +137,8 @@ void drawLabel::bounds(bbox& b, iopipestream& tex, boxvector& labelbounds,
 	  break;
 	}
 
-	pair Align=(align == pair(0,0)) ? pair(1,0) : unit(align);
+	pair Align=(align == pair(0,0)) ? unit(pair(urand(),urand())) :
+	  unit(align);
 	double s=0.1*pentype.size();
 	double dx=0, dy=0;
 	if(Align.getx() > 0.1) dx=labelbounds[i].xmax()-Box.xmin()+s;
@@ -171,12 +170,8 @@ void drawLabel::bounds(bbox& b, iopipestream& tex, boxvector& labelbounds,
 
 drawElement *drawLabel::transformed(const transform& t)
 {
-  static const pair origin=pair(0,0);
-  pair offset=t*origin;
-  return new drawLabel(label,size,
-		       degrees((t*expi(radians(angle))-offset).angle()),
-		       t*position,length(align)*unit(t*align-offset),scale,
-		       pentype);
+  return new drawLabel(label,size,t*T,t*position,
+		       length(align)*unit(shiftless(t)*align),pentype);
 }
 
 } //namespace camp
