@@ -82,9 +82,9 @@ types::ty *tempExp::trans(coenv &e) {
 
 
 varEntryExp::varEntryExp(position pos, types::ty *t, access *a)
-  : exp(pos), v(new trans::varEntry(t, a)) {}
+  : exp(pos), v(new trans::varEntry(t, a, 0)) {}
 varEntryExp::varEntryExp(position pos, types::ty *t, vm::bltin f)
-  : exp(pos), v(new trans::varEntry(t, new bltinAccess(f))) {}
+  : exp(pos), v(new trans::varEntry(t, new bltinAccess(f), 0)) {}
 
 types::ty *varEntryExp::getType(coenv &) {
   return v->getType();
@@ -415,7 +415,7 @@ void argument::prettyprint(ostream &out, int indent)
   out << "explist";
   if (name)
     out << " '" << *name << "'";
-  out << endl;
+  out << '\n';
 
   val->prettyprint(out, indent+1);
 }
@@ -481,17 +481,27 @@ application *callExp::resolve(coenv &e, overloaded *o, signature *source) {
   if (l.empty()) {
     //cerr << "l is empty\n";
     em->error(getPos());
+
     symbol *s = callee->getName();
     if (s)
-      *em << "no matching function \'" << *s << *source << "\'";
+      *em << "no matching function \'" << *s;
     else
-      *em << "no matching function for signature \'" << *source << "\'";
+      *em << "no matching function for signature \'";
+    *em << *source << "\'";
+
     return 0;
   }
   else if (l.size() > 1) { // This may take O(n) time.
     //cerr << "l is full\n";
     em->error(getPos());
-    *em << "call with signature \'" << *source << "\' is ambiguous";
+
+    symbol *s = callee->getName();
+    if(s)
+      *em << "call of function \'" << *s;
+    else
+      *em << "call with signature \'";
+    *em << *source << "\' is ambiguous";
+
     return 0;
   }
   else {
@@ -534,6 +544,8 @@ application *callExp::getApplication(coenv &e)
     for (size_t i = 0; i < args->size(); i++) {
       (*args)[i].val->trans(e);
     }
+    if (args->rest.val)
+      args->rest.val->trans(e);
     return 0;
   }
 
@@ -695,26 +707,6 @@ types::ty *transformExp::trans(coenv &e)
   return types::primTransform();
 }
 
-void dimensions::prettyprint(ostream &out, int indent)
-{
-  prettyindent(out, indent);
-  out << "dimensions (" << depth << ")\n";
-}
-
-types::ty *dimensions::truetype(types::ty *base)
-{
-  if (base->kind == ty_void) {
-    em->compiler(getPos());
-    *em << "can't declare array of type void";
-    return primError();
-  }
-  for (size_t d = depth; d > 0; d--) {
-    base = new types::array(base);
-  }
-  return base;
-}
-
-
 void castExp::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "castExp",indent);
@@ -953,7 +945,7 @@ void specExp::prettyprint(ostream &out, int indent)
   out << "specExp '" << *op << "' " 
       << (s==camp::OUT ? "out" :
           s==camp::IN  ? "in" :
-                         "invalid side") << endl;
+	  "invalid side") << '\n';
 
   arg->prettyprint(out, indent+1);
 }
