@@ -206,7 +206,7 @@ public:
     if(csvmode) {
       val=whitespace+getcsvline();
     } else {
-      mem::string s=mem::string();
+      mem::string s;
       getline(*stream,s);
       val=whitespace+s;
     }
@@ -268,10 +268,21 @@ public:
   void write(guide *val) {*stream << *val;}
   void write(const transform& val) {*stream << val;}
   void writeline() {
-    if(standard && settings::scrollLines) {
-      if(lines > 0 && lines % settings::scrollLines == 0) {
-	while(std::cin.good() && std::cin.get() != '\n') continue;
-	if(!std::cin.good()) *stream << newline;
+    if(standard && interact::interactive) {
+      int scroll=settings::getScroll();
+      if(scroll && lines > 0 && lines % scroll == 0) {
+	for(;;) {
+	  if(!std::cin.good()) {
+	    *stream << newline;
+	    std::cin.clear();
+	    break;
+	  }
+	  int c=std::cin.get();
+	  if(c == '\n') break;
+	  // Discard any additional characters
+	  while(std::cin.good() && std::cin.get() != '\n');
+	  if(c == 'q') throw quit();
+	}
       } else *stream << newline;
       ++lines;
     } else *stream << newline;
@@ -350,7 +361,6 @@ public:
 
 extern ofile Stdout;
 extern ofile nullfile;
-extern string asyinput;
 
 template<class T>
 void ifile::iread(T& val)
