@@ -36,16 +36,22 @@ types::formal formal::trans(coenv &e, bool encodeDefVal, bool tacit) {
 }
 
 types::ty *formal::getType(coenv &e, bool tacit) {
-  types::ty *t = start ? start->getType(base->trans(e), e, tacit)
-    : base->trans(e, tacit);
+  types::ty *bt = base->trans(e, tacit);
+  types::ty *t = start ? start->getType(bt, e, tacit) : bt;
   if (t->kind == ty_void && !tacit) {
     em->compiler(getPos());
-    *em << "can't declare parameters of type void";
+    *em << "cannot declare parameters of type void";
     return primError();
   }
+
   return t;
 }
   
+void formal::addOps(coenv &e) {
+  if (start)
+    start->addOps(base->trans(e, true), e, (record *)0);
+} 
+
 void formals::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "formals",indent);
@@ -135,7 +141,12 @@ void formal::transAsVar(coenv &e, int index) {
     // Suppress error messages because they will already be reported
     // when the formals are translated to yield the type earlier.
     types::ty *t = getType(e, true);
-    varEntry *v = new varEntry(t, a);
+    varEntry *v = new varEntry(t, a, 0);
+
+    // Add operations if a new array type is formed.  This will add to the
+    // environment at the start of the function, when the formals are also added
+    // to the environment.
+    addOps(e);
 
     // Translate the default argument before adding the formal to the
     // environment, consistent with the initializers of variables.
@@ -221,21 +232,6 @@ void fundec::prettyprint(ostream &out, int indent)
   fun.prettyprint(out, indent);
 }
 
-function *fundec::opType(function *f)
-{
-  return new function(primBoolean(),types::formal(f,"a"),types::formal(f,"b"));
-
-}
-
-void fundec::addOps(coenv &e, record *r, function *f)
-{
-  function *ft = opType(f);
-  addVar(e,r,new varEntry(ft, new bltinAccess(run::boolFuncEq)),
-	 symbol::trans("=="));
-  addVar(e,r,new varEntry(ft, new bltinAccess(run::boolFuncNeq)),
-	 symbol::trans("!="));
-}
-
 void fundec::trans(coenv &e)
 {
   transAsField(e,0);
@@ -246,7 +242,9 @@ void fundec::transAsField(coenv &e, record *r)
   function *ft = fun.transType(e, true);
   assert(ft);
 
-  addOps(e,r,ft);
+  e.e.addFunctionOps(ft);
+  if (r)
+    r->e.addFunctionOps(ft);
   
   createVar(getPos(), e, r, id, ft, &fun);
 } 
