@@ -79,6 +79,16 @@ quadraticroots::quadraticroots(double a, double b, double c)
   }
 }
 
+inline bool goodroot(double t)
+{
+  return 0.0 <= t && t <= 1.0;
+}
+
+inline bool goodroot(double a, double b, double c, double t)
+{
+  return goodroot(t) && quadratic(a,b,c,t) >= 0.0;
+}
+
 // Accurate computation of cbrt(sqrt(1+x)+1)-cbrt(sqrt(1+x)-1).
 inline double cbrtsqrt1pxm(double x)
 {
@@ -125,23 +135,25 @@ cubicroots::cubicroots(double a, double b, double c, double d)
   double R2=R*R;
   double D=Q3+R2;
   double mthirdb=-b*third;
-  double x=Q3/R2;
   
   if(D > 0.0) {
     roots=1;
     t1=mthirdb;
-    if(R != 0.0) t1 += cbrt(R)*cbrtsqrt1pxm(x);
+    if(R2 != 0.0) t1 += cbrt(R)*cbrtsqrt1pxm(Q3/R2);
   } else {
     roots=3;
-    double u=-x-1;
-    double v=u >= 0.0 ? sqrt(u) : 0.0;
-    double theta=atan(v);
-    double factor=2*cbrt(sqrt(-x)*R);
+    double v=0.0,theta;
+    if(R2 > 0.0) {
+      v=sqrt(-D/R2);
+      theta=atan(v);
+    } else theta=0.5*PI;
+    double factor=2.0*sqrt(-Q)*(R >= 0 ? 1 : -1);
       
     t1=mthirdb+factor*cos(third*theta);
     t2=mthirdb-factor*cos(third*(theta-PI));
-    t3=mthirdb-factor*
-      ((v < 100.0) ? cos(third*(theta+PI)) : costhetapi3(1/v)); 
+    t3=mthirdb;
+    if(R2 > 0.0)
+      t3 -= factor*((v < 100.0) ? cos(third*(theta+PI)) : costhetapi3(1.0/v)); 
   }
 }
   
@@ -253,7 +265,8 @@ pair path::postcontrol(double t) const
 path path::reverse() const
 {
   solvedKnot *nodes = new solvedKnot[n];
-  for (int i = 0, j = length(); i < n; i++, j--) {
+  int len=length();
+  for (int i = 0, j = len; i < n; i++, j--) {
     nodes[i].pre = postcontrol(j);
     nodes[i].point = point(j);
     nodes[i].post = precontrol(j);
@@ -268,7 +281,8 @@ path path::subpath(int start, int end) const
 
   if (start > end) {
     const path &rp = reverse();
-    path result = rp.subpath(length()-start, length()-end);
+    int len=length();
+    path result = rp.subpath(len-start, len-end);
     return result;
   }
 
@@ -314,16 +328,22 @@ path path::subpath(double start, double end) const
   
   if (start > end) {
     const path &rp = reverse();
-    path result = rp.subpath(length()-start, length()-end);
-    return result;
+    int len=length();
+    return rp.subpath(len-start, len-end);
   }
 
   solvedKnot startL, startR, endL, endR;
   if (!cycles) {
-    if (start < 0)
+    if (start < 0) {
       start = 0;
-    if (end > n-1)
+      if (end < 0)
+	end = 0;
+    }	
+    if (end > n-1) {
       end = n-1;
+      if (start > n-1)
+	start = n-1;
+    }
     startL = nodes[(int)floor(start)];
     startR = nodes[(int)ceil(start)];
     endL = nodes[(int)floor(end)];
@@ -375,7 +395,8 @@ bbox path::bounds() const
 
   if(!box.empty) return box;
   
-  for (int i = 0; i < length(); i++) {
+  int len=length();
+  for (int i = 0; i < len; i++) {
     box += point(i);
     if(straight(i)) continue;
     
@@ -384,15 +405,94 @@ bbox path::bounds() const
     
     // Check x coordinate
     quadraticroots x(a.getx(),b.getx(),c.getx());
-    if(x.distinct != quadraticroots::NONE) box += point(i+x.t1);
-    if(x.distinct == quadraticroots::TWO) box += point(i+x.t2);
+    if(x.distinct != quadraticroots::NONE && goodroot(x.t1))
+      box += point(i+x.t1);
+    if(x.distinct == quadraticroots::TWO && goodroot(x.t2))
+      box += point(i+x.t2);
     
     // Check y coordinate
     quadraticroots y(a.gety(),b.gety(),c.gety());
-    if(y.distinct != quadraticroots::NONE) box += point(i+y.t1);
-    if(y.distinct == quadraticroots::TWO) box += point(i+y.t2);
+    if(y.distinct != quadraticroots::NONE && goodroot(y.t1))
+      box += point(i+y.t1);
+    if(y.distinct == quadraticroots::TWO && goodroot(y.t2))
+      box += point(i+y.t2);
   }
-  box += point(length());
+  box += point(len);
+  return box;
+}
+
+bbox path::bounds(const bbox& padding) const
+{
+  if (empty()) {
+    // No bounds
+    return bbox(/* empty */);
+  }
+
+  if(!box.empty) return box;
+  
+  if(cyclic()) return pad(bounds(),padding);
+  
+  box += point(0);
+  
+  // Check interior nodes.
+  int len=length();
+  for (int i = 1; i < len; i++) {
+    box += point(i);
+    
+    pair pre=point(i)-precontrol(i);
+    pair post=postcontrol(i)-point(i);
+    
+    // Check node x coordinate
+    if(pre.getx() >= 0.0 ^ post.getx() >= 0) {
+      pair z=point(i);
+      box += z+padding.left;
+      box += z+padding.right;
+    }
+			      
+    // Check node y coordinate
+    if(pre.gety() >= 0.0 ^ post.gety() >= 0) {
+      pair z=point(i);
+      box += z+pair(0,padding.bottom);
+      box += z+pair(0,padding.top);
+    }
+  }
+			      
+  // Check interior segments.
+  for (int i = 0; i < len; i++) {
+    if(straight(i)) continue;
+    
+    pair a,b,c;
+    derivative(a,b,c,point(i),postcontrol(i),precontrol(i+1),point(i+1));
+    
+    // Check x coordinate
+    quadraticroots x(a.getx(),b.getx(),c.getx());
+    if(x.distinct != quadraticroots::NONE && goodroot(x.t1)) {
+      pair z=point(i+x.t1);
+      box += z+padding.left;
+      box += z+padding.right;
+    }
+    if(x.distinct == quadraticroots::TWO && goodroot(x.t2)) {
+      pair z=point(i+x.t2);     
+      box += z+padding.left;
+      box += z+padding.right;
+    }
+    
+    // Check y coordinate
+    quadraticroots y(a.gety(),b.gety(),c.gety());
+    if(y.distinct != quadraticroots::NONE && goodroot(y.t1)) {
+      pair z=point(i+y.t1);     
+      box += z+pair(0,padding.bottom);
+      box += z+pair(0,padding.top);
+    }
+    if(y.distinct == quadraticroots::TWO && goodroot(y.t2)) {
+      pair z=point(i+y.t2);
+      box += z+pair(0,padding.bottom);
+      box += z+pair(0,padding.top);
+    }
+    
+  }
+  
+  box += point(len);
   return box;
 }
 
@@ -493,11 +593,6 @@ double path::arctime(double goal) const {
 
 // }}}
 
-inline bool goodroot(double a, double b, double c, double t)
-{
-  return 0.0 <= t && t <= 1.0 && quadratic(a,b,c,t) >= 0.0;
-}
-
 // {{{ Direction Time Calulation
 // Algorithm Stolen from Knuth's MetaFont
 inline double cubicDir(const solvedKnot& left, const solvedKnot& right,
