@@ -1,4 +1,3 @@
-
 /*****
  * picture.cc
  * Andy Hammerlindl 2002/06/06
@@ -20,10 +19,10 @@ using std::ofstream;
 
 using namespace settings;
 
-double pageWidth, pageHeight;
-
 namespace camp {
 
+const char *texpathmessage="the directory containing your tex engine (latex by default)";
+  
 string texready=string("(Please type a command or say `\\end')\n*");
 texstream tex; // Bi-directional pipe to latex (to find label bbox)
 
@@ -31,6 +30,7 @@ void texstream::pipeclose() {
   iopipestream::pipeclose();
   if (!getSetting<bool>("keep")) {
     unlink("texput.log");
+    unlink("texput.out");
     unlink("texput.aux");
   }
 }
@@ -39,6 +39,10 @@ picture::~picture()
 {
 }
 
+bool picture::epsformat,picture::pdfformat,picture::tgifformat, picture::pdf;
+bool picture::Labels;
+double picture::paperWidth,picture::paperHeight;
+  
 void picture::enclose(drawElement *begin, drawElement *end)
 {
   assert(begin);
@@ -92,7 +96,7 @@ void picture::prepend(picture &pic)
 bool picture::havelabels()
 {
   size_t n=nodes.size();
-  if(n > lastnumber && !labels && getSetting<bool>("tex")) {
+  if(n > lastnumber && !labels && getSetting<mem::string>("tex") != "none") {
     // Check to see if there are any labels yet
     nodelist::iterator p=nodes.begin();
     for(size_t i=0; i < lastnumber; ++i) ++p;
@@ -140,7 +144,8 @@ void picture::texinit()
     return;
   }
   
-  tex.open(getSetting<mem::string>("latex").c_str(),"latex","latex");
+
+  tex.open(texengine().c_str(),"texpath",texpathmessage);
   texdocumentclass(tex,true);
   
   texdefines(tex,TeXpipepreamble,true);
@@ -152,7 +157,7 @@ void picture::texinit()
 }
   
 bool picture::texprocess(const string& texname, const string& outname,
-			 const string& prefix, const bbox& bpos) 
+			 const string& prefix, const pair& bboxshift) 
 {
   int status=0;
   ifstream outfile;
@@ -161,136 +166,135 @@ bool picture::texprocess(const string& texname, const string& outname,
   if(outfile) {
     outfile.close();
     ostringstream cmd;
-    cmd << "'" << getSetting<mem::string>("latex") << "'"
-	<< " \\scrollmode\\input " << texname;
+    cmd << "'" << texengine() << "'"
+	<< " \\scrollmode\\input '" << texname << "'";
     bool quiet=verbose <= 1;
-    status=System(cmd,quiet,"latex");
+    status=System(cmd,quiet ? 1 : 0,"texpath",texpathmessage);
+    if(!status && getSetting<bool>("twice")) 
+      status=System(cmd,quiet ? 1 : 0,"texpath",texpathmessage);
     if(status) {
-      if(quiet) status=System(cmd,false,"latex");
+      if(quiet) System(cmd,0);
       return false;
     }
     
-    string dviname=auxname(prefix,"dvi");
-    string psname=auxname(prefix,"ps");
+    mem::string texengine=getSetting<mem::string>("tex");
+    if(settings::pdf(texengine)) {
+      if(pdfformat) {
+	rename(outname.c_str(),buildname(prefix,"pdf").c_str());
+      }
+    } else {
+      string dviname=auxname(prefix,"dvi");
+      string psname=auxname(prefix,"ps");
     
-    double height=bpos.top-bpos.bottom;
-    double width=bpos.right-bpos.left;
+      double height=b.top-b.bottom+1.0;
     
-    // Magic dvips offsets:
-    double hoffset=-128.0;
-    double voffset=(height < 11.0) ? -137.0+height : -126.0;
-    
-    int origin=getSetting<int>("align");
+      // Magic dvips offsets:
+      double hoffset=-128.4;
+      double vertical=height;
+      if(!settings::latex(texengine)) vertical += 2.0;
+      double voffset=(vertical < 13.0) ? -137.8+vertical : -124.8;
 
-    if(origin != ZERO) {
-      if(pdfformat || origin == BOTTOM) {
-	voffset += max(pageHeight-(height+1.0),0.0);
-      } else if(origin == CENTER) {
-	hoffset += 0.5*max(pageWidth-(width+1.0),0.0);
-	voffset += 0.5*max(pageHeight-(height+1.0),0.0);
+      hoffset += b.left+bboxshift.getx();
+      voffset += paperHeight-height-b.bottom-bboxshift.gety();
+    
+      ostringstream dcmd;
+      dcmd << "'" << getSetting<mem::string>("dvips") << "' -R "
+	   << " -O " << hoffset << "bp," << voffset << "bp"
+	   << " -T " << paperWidth << "bp," << paperHeight << "bp";
+      if(verbose <= 1) dcmd << " -q";
+      dcmd << " -o '" << psname << "' '" << dviname << "'";
+      status=System(dcmd,0,true,"dvips");
+    
+      ifstream fin(psname.c_str());
+      psfile fout(outname,false);
+    
+      string s;
+      bool first=true;
+      transform t=shift(bboxshift);
+      if(T) t=t*(*T);
+      bool shift=(t != identity());
+      string beginspecial="TeXDict begin @defspecial";
+      string endspecial="@fedspecial end";
+      while(getline(fin,s)) {
+	if(s.find("%%DocumentPaperSizes:") == 0) continue;
+	if(first && s.find("%%BoundingBox:") == 0) {
+	  bbox box=b;
+	  box.shift(bboxshift);
+	  if(verbose > 2) BoundingBox(cout,box);
+	  fout.BoundingBox(box);
+	  first=false;
+	} else if(shift && s.find(beginspecial) == 0) {
+	  fout.verbatimline(s);
+	  fout.gsave();
+	  fout.concat(t);
+	} else if(shift && s.find(endspecial) == 0) {
+	  fout.grestore();
+	  fout.verbatimline(s);
+	} else
+	  fout.verbatimline(s);
+      }
+      if(!getSetting<bool>("keep")) { // Delete temporary files.
+	unlink(dviname.c_str());
+	unlink(psname.c_str());
       }
     }
-    
-    if(!pdfformat) {
-      hoffset += getSetting<pair>("offset").getx();
-      voffset -= getSetting<pair>("offset").gety();
-    }
-
-    ostringstream dcmd;
-    dcmd << "'" << getSetting<mem::string>("dvips") << "' -R -t " 
-	 << getSetting<mem::string>("papertype") 
-	 << "size -O " << hoffset << "bp," << voffset << "bp";
-    if(verbose <= 1) dcmd << " -q";
-    dcmd << " -o " << psname << " " << dviname;
-    status=System(dcmd,false,true,"dvips");
-    
-    bbox bcopy=bpos;
-    double hfuzz=0.1;
-    double vfuzz=0.2;
-    if(origin == CENTER || origin == TOP) {
-      hfuzz *= 2.0; vfuzz *= 2.0;
-    }
-    
-    bcopy.left -= hfuzz;
-    bcopy.right += hfuzz;
-    
-    bcopy.bottom -= vfuzz;
-    bcopy.top += vfuzz;
-    
-    ifstream fin(psname.c_str());
-    ofstream *Fout=NULL;
-    ostream *fout=(outname == "") ? &cout :
-      Fout=new ofstream(outname.c_str());
-    
-    string s;
-    bool first=true;
-    while(getline(fin,s)) {
-      if(s.find("%%DocumentPaperSizes:") == 0) continue;
-      if(first && s.find("%%BoundingBox:") == 0) {
-	if(verbose > 2) BoundingBox(cout,bpos);
-	BoundingBox(*fout,bcopy);
-	first=false;
-      } else *fout << s << endl;
-    }
-    if(Fout) {
-      if(!Fout->good()) {
-	ostringstream msg;
-	msg << "Cannot write to " << outname.c_str();
-	reportError(msg);
-      }
-      delete Fout;
-    }
-    
+      
     if(!getSetting<bool>("keep")) { // Delete temporary files.
       unlink(texname.c_str());
-      unlink(dviname.c_str());
-      unlink(psname.c_str());
       unlink(auxname(prefix,"aux").c_str());
       unlink(auxname(prefix,"log").c_str());
+      unlink(auxname(prefix,"out").c_str());
     }
   }
   if(status) return false;
   return true;
 }
 
-bool picture::postprocess(const string& epsname, const string& outname,
-			  const string& outputformat, bool wait, bool quiet,
-			  const bbox& bpos)
+int picture::epstopdf(const string& epsname, const string& pdfname)
 {
-  int status=0;
   ostringstream cmd;
   
-  if(!epsformat) {
+  cmd << "'" << getSetting<mem::string>("gs")
+      << "' -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -dEPSCrop"
+      << " -dCompatibilityLevel=1.4";
+  if(!getSetting<bool>("autorotate"))
+    cmd << " -dAutoRotatePages=/None";
+  cmd << " -g" << ceil(paperWidth) << "x" << ceil(paperHeight)
+      << " -dDEVICEWIDTHPOINTS=" << b.right-b.left
+      << " -dDEVICEHEIGHTPOINTS=" << b.top-b.bottom
+      << " -sOutputFile='" << pdfname << "' '" << epsname << "'";
+  return System(cmd,0,true,"gs","Ghostscript");
+}
+  
+bool picture::postprocess(const string& prename, const string& outname,
+			  const string& outputformat, bool wait, bool view)
+{
+  int status=0;
+  
+  if((pdf && Labels) || !epsformat) {
     if(pdfformat) {
-      cmd << "'" << getSetting<mem::string>("gs")
-	  << "' -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -dEPSCrop"
-	  << " -dAutoRotatePages=/None "
-	  << " -dDEVICEWIDTHPOINTS=" 
-	  << bpos.right-bpos.left+1.0
-	  << " -dDEVICEHEIGHTPOINTS=" 
-	  << bpos.top-bpos.bottom+1.0
-	  << " -sOutputFile=" << outname << " " << epsname;
-      status=System(cmd,false,true,"gs","Ghostscript");
+      if(!(pdf && Labels)) status=epstopdf(prename,outname);
     } else {
+      ostringstream cmd;
       double expand=2.0;
       double res=(tgifformat ? getSetting<double>("deconstruct") : expand)*
 	72.0;
       cmd << "'" << getSetting<mem::string>("convert") 
 	  << "' -density " << res << "x" << res;
       if(!tgifformat) cmd << " +antialias -geometry " << 100.0/expand << "%x";
-      cmd << " eps:" << epsname;
+      cmd << " '" << (pdf ? "pdf:" : "eps:") << prename << "'";
       if(tgifformat) cmd << " -transparent white gif";
       else cmd << " " << outputformat;
-      cmd << ":" << outname;
-      status=System(cmd,false,true,"convert");
+      cmd << ":'" << outname << "'";
+      status=System(cmd,0,true,"convert");
     }
-    if(!getSetting<bool>("keep")) unlink(epsname.c_str());
+    if(!getSetting<bool>("keep")) unlink(prename.c_str());
   }
   if(status != 0) return false;
   
   if(verbose > (tgifformat ? 1 : 0)) 
     cout << "Wrote " << outname << endl;
-  if(view() && !quiet) {
+  if(settings::view() && view) {
     if(epsformat || pdfformat) {
       static int pid=0;
       static string lastoutname;
@@ -300,14 +304,14 @@ bool picture::postprocess(const string& epsname, const string& outname,
       if(interact::interactive && pid)
 	restart=(waitpid(pid, &status, WNOHANG) == pid);
 
-      if (!interact::virtualEOF || outname != lastoutname || restart) {
+      if (outname != lastoutname || restart) {
 	if(!wait) lastoutname=outname;
 	ostringstream cmd;
 	cmd << "'" << Viewer << "'";
 	if(Viewer == "gv" && interact::interactive)
-	  cmd << " -nowatch";
-	cmd << " " << outname;
-	status=System(cmd,false,wait,
+	  cmd << " " << gvOptionPrefix << "nowatch";
+	cmd << " '" << outname << "'";
+	status=System(cmd,0,wait,
 		      pdfformat ? "pdfviewer" : "psviewer",
 		      pdfformat ? "your PDF viewer" : "your PostScript viewer",
 		      &pid);
@@ -315,9 +319,10 @@ bool picture::postprocess(const string& epsname, const string& outname,
       } else if(Viewer == "gv") kill(pid,SIGHUP); // Tell gv to reread file.
     } else {
       ostringstream cmd;
-      cmd << "'" << getSetting<mem::string>("display") << "' " << outname;
+      cmd << "'" << getSetting<mem::string>("display") << "' '"
+	  << outname << "'";
       string application="your "+outputformat+" viewer";
-      status=System(cmd,false,wait,"display",application.c_str());
+      status=System(cmd,0,wait,"display",application.c_str());
       if(status != 0) return false;
     }
   }
@@ -326,100 +331,109 @@ bool picture::postprocess(const string& epsname, const string& outname,
 }
 
 bool picture::shipout(picture *preamble, const string& Prefix,
-		      const string& format, bool wait, bool quiet, bool Delete)
+		      const string& format, bool wait, bool view, bool Delete)
 {
+  bounds();
+  
+  bool TeXmode=getSetting<bool>("inlinetex") && 
+    getSetting<mem::string>("tex") != "none";
+  Labels=(labels || TeXmode) && (b.right > b.left && b.top > b.bottom);
+  
+  pdf=settings::pdf(getSetting<mem::string>("tex"));
+  
   bool standardout=Prefix == "-";
   string prefix=standardout ? "out" : Prefix;
   checkFormatString(format);
-  string outputformat=format.empty() ? 
-    (string)getSetting<mem::string>("outformat") : format;
-  epsformat=outputformat.empty() || outputformat == "eps";
+  string preformat=pdf ? "pdf" : "eps";
+  string defaultformat=(string) getSetting<mem::string>("outformat");
+  if(defaultformat == "") defaultformat=preformat;
+  string outputformat=format.empty() ? defaultformat : format;
+  epsformat=outputformat == "eps";
   pdfformat=outputformat == "pdf";
   tgifformat=outputformat == "tgif";
   string outname=tgifformat ? "."+buildname(prefix,"gif") :
     (standardout ? "-" : buildname(prefix,outputformat,"",false));
   string epsname=epsformat ? (standardout ? "" : outname) :
     auxname(prefix,"eps");
+  string prename=((epsformat && !pdf) || !Labels) ? epsname : 
+    auxname(prefix,preformat);
   double deconstruct=getSetting<double>("deconstruct");
   
-  bounds();
-  
   static ofstream bboxout;
   
   if(null()) { // Output a null file
     bbox b;
     b.left=b.bottom=0;
     b.right=b.top=1;
-    psfile out(epsname,b,0);
-    out.prologue();
+    psfile out(epsname,false);
+    out.prologue(b);
     out.epilogue();
     if(deconstruct && !tgifformat) {
       if(bboxout) bboxout.close();
       ShipoutNumber++;
       return true;
     }
-    return postprocess(epsname,outname,outputformat,wait,quiet,b);
+    return postprocess(epsname,outname,outputformat,wait,view);
   }
   
   if(deconstruct && !tgifformat) {
     if(bboxout) bboxout.close();
-    if(view()) {
+    if(settings::view() && view) {
       ostringstream cmd;
       string Python=getSetting<mem::string>("python");
       if(Python != "") cmd << "'" << Python << "' ";
       cmd << "'" << getSetting<mem::string>("xasy") << "' " 
 	  << buildname(prefix) << " " << ShipoutNumber << " "
-	  << buildname(getSetting<mem::string>("outname"));
-      int status=System(cmd,false,true,Python != "" ? "python" : "xasy");
+	  << buildname(settings::outname());
+      int status=System(cmd,0,true,Python != "" ? "python" : "xasy");
       if(status != 0) return false;
     }
     ShipoutNumber++;
     return true;
   }
       
-  bbox bpos=b;
-  
-  bool TeXmode=getSetting<bool>("inlinetex") && getSetting<bool>("tex");
-  bool Labels=labels || TeXmode;
-  
   if(deconstruct) {
       if(!bboxout.is_open()) {
 	bboxout.open(("."+buildname(prefix,"box")).c_str());	
-	bboxout << deconstruct << endl;
+	bboxout << deconstruct << newl;
       }
       bbox bscaled=b;
       bscaled *= deconstruct;
-      bboxout << bscaled << endl;
+      bboxout << bscaled << newl;
       if(Delete) {
 	unlink(outname.c_str());
 	return false;
       }
   }
   
-  GetPageDimensions(pageWidth,pageHeight);
+  SetPageDimensions();
   
-  // Avoid negative bounding box coordinates
+  paperWidth=getSetting<double>("paperwidth");
+  paperHeight=getSetting<double>("paperheight");
   int origin=getSetting<int>("align");
-  bboxshift=origin == ZERO ? 0.0 : pair(-bpos.left,-bpos.bottom);
+    
+  pair bboxshift=(origin == ZERO && !pdfformat) ?
+    pair(0.0,0.0) : pair(-b.left,-b.bottom);
   if(!pdfformat) {
     bboxshift += getSetting<pair>("offset");
-    if(!(origin == BOTTOM || origin == ZERO)) {
-      double yexcess=max(pageHeight-(bpos.top-bpos.bottom),0.0);
-      if(origin == TOP) bboxshift += pair(0.5,yexcess-0.5);
+    if(origin != ZERO && origin != BOTTOM) {
+      double yexcess=max(paperHeight-(b.top-b.bottom+1.0),0.0);
+      if(origin == TOP) bboxshift += pair(0.0,yexcess);
       else {
-	double xexcess=max(pageWidth-(bpos.right-bpos.left),0.0);
-	bboxshift += 0.5*pair(xexcess,yexcess);
+	double xexcess=max(paperWidth-(b.right-b.left+1.0),0.0);
+	bboxshift += pair(0.5*xexcess,0.5*yexcess);
       }
     }
   }
-  bpos.shift(bboxshift);
   
   bool status = true;
   
-  string texname=auxname(prefix,"tex");
+  string texname;
   texfile *tex=NULL;
   
   if(Labels) {
+    spaceToUnderscore(prefix);
+    texname=auxname(prefix,"tex");
     tex=new texfile(texname,b);
     tex->prologue();
   }
@@ -429,15 +443,28 @@ bool picture::shipout(picture *preamble, const string& Prefix,
   unsigned int layer=0;
   std::list<string> psnameStack;
   
-  while(p != nodes.end()) {
-    ostringstream buf;
-    buf << prefix << "_" << layer;
-    string psname=Labels ? buildname(buf.str(),"eps") : epsname;
-    psnameStack.push_back(psname);
-    psfile out(psname,bpos,bboxshift);
-    out.prologue();
+  bbox bshift=b;
   
-    if(Labels) tex->beginlayer(psname);
+  while(p != nodes.end()) {
+    string psname,pdfname;
+    if(Labels) {
+      ostringstream buf;
+      buf << prefix << "_" << layer;
+      psname=buildname(buf.str(),"eps");
+      if(pdf) pdfname=buildname(buf.str(),"pdf");
+    } else {
+      psname=epsname;
+      bshift.shift(bboxshift);
+    }
+    psnameStack.push_back(psname);
+    psfile out(psname,pdfformat);
+    out.prologue(bshift);
+  
+    if(Labels) tex->beginlayer(pdf ? pdfname : psname);
+    else {
+      out.gsave();
+      out.translate(bboxshift);
+    }
   
     if(preamble) {
       // Postscript preamble.
@@ -447,10 +474,8 @@ bool picture::shipout(picture *preamble, const string& Prefix,
 	out.resetpen();
 	for(; P != Nodes.end(); ++P) {
 	  assert(*P);
-	  out.raw(true);
 	  if(!(*P)->draw(&out))
 	    status = false;
-	  out.raw(false);
 	}
       }
     }
@@ -462,43 +487,56 @@ bool picture::shipout(picture *preamble, const string& Prefix,
       if(!(*p)->draw(&out))
 	status = false;
     }
+    if(!Labels) out.grestore();
+    
     out.epilogue();
-  
-    if(status && Labels) {
-      for (p=layerp; p != nodes.end(); ++p) {
-	if((*p)->islayer()) {
-	  tex->endlayer();
-	  layerp=++p;
-	  layer++;
-	  break;
+    
+    if(Labels) {
+      if(status) {
+	if(pdf) {
+	  out.close();
+	  status=(epstopdf(psname,pdfname) == 0);
 	}
-	assert(*p);
-	if(!(*p)->write(tex))
-	  status = false;
-      }
-    }    
+	
+	if(status) {
+	  for (p=layerp; p != nodes.end(); ++p) {
+	    if((*p)->islayer()) {
+	      tex->endlayer();
+	      layerp=++p;
+	      layer++;
+	      break;
+	    }
+	    assert(*p);
+	    if(!(*p)->write(tex))
+	      status = false;
+	  }
+	}
+      }    
+    }
   }
   
   if(status) {
     if(TeXmode) {
-      if(verbose > 0) cout << "Wrote " << texname << endl;
+      if(Labels && verbose > 0) cout << "Wrote " << texname << endl;
+      delete tex;
     } else {
-      if(labels) {
+      if(Labels) {
 	tex->epilogue();
-	status=texprocess(texname,epsname,prefix,bpos);
-	if(!getSetting<bool>("keep"))
+	status=texprocess(texname,prename,prefix,bboxshift);
+	delete tex;
+	if(!getSetting<bool>("keep")) {
 	  for(std::list<string>::iterator p=psnameStack.begin();
 	      p != psnameStack.end(); ++p)
 	    unlink(p->c_str());
+	}
       }
       if(status)
-	status=postprocess(epsname,outname,outputformat,wait,quiet,bpos);
+	status=postprocess(prename,outname,outputformat,wait,view);
     }
   }
   
   if(!status) reportError("shipout failed");
     
-  delete tex;
   return status;
 }
 
@@ -511,6 +549,7 @@ picture *picture::transformed(const transform& t)
     assert(*p);
     pic->append((*p)->transformed(t));
   }
+  pic->T=new transform(T ? t*(*T) : t);
 
   return pic;
 }
