@@ -22,26 +22,56 @@ using vm::read;
   
 namespace camp {
 
-psfile::psfile(const string& filename, const bbox& box, const pair& Shift)
-  : filename(filename), box(box), Shift(Shift), rawmode(false)
+void checkColorSpace(ColorSpace colorspace)
+{
+  switch(colorspace) {
+  case DEFCOLOR:
+  case INVISIBLE:
+    reportError("Cannot shade with invisible pen");
+  case PATTERN:
+    reportError("Cannot shade with pattern");
+    break;
+  default:
+    break;
+  }
+}
+    
+psfile::psfile(const string& filename, bool pdfformat)
+  : filename(filename), pdfformat(pdfformat), pdf(false), out(NULL)
 {
   if(filename.empty()) out=&std::cout;
   else out=new ofstream(filename.c_str());
   if(!out || !*out) {
-    std::cerr << "Can't write to " << filename << std::endl;
+    std::cerr << "Cannot write to " << filename << std::endl;
     throw handled_error();
   }
 }
 
+void psfile::close()
+{
+  if(out) {
+    out->flush();
+    if(!filename.empty()) {
+      if(!out->good()) {
+	std::ostringstream msg;
+	msg << "Cannot write to " << filename;
+	reportError(msg);
+      }
+      delete out;
+      out=NULL;
+    }
+  }
+}
+
 psfile::~psfile()
 {
-  if(!filename.empty() && out) delete out;
+  close();
 }
-
-void psfile::prologue()
+  
+void psfile::prologue(const bbox& box)
 {
   *out << "%!PS-Adobe-3.0 EPSF-3.0" << newl;
-  BoundingBox(*out,box);
+  BoundingBox(box);
   *out << "%%Creator: " << settings::PROGRAM << " " << settings::VERSION
        <<  newl;
 
@@ -63,34 +93,47 @@ void psfile::epilogue()
 {
   *out << "showpage" << newl;
   *out << "%%EOF" << newl;
-  out->flush();
 }
 
+void psfile::setcolor(const pen& p, const string& begin="",
+		      const string& end="")
+{
+  if(p.cmyk() && (!lastpen.cmyk() ||
+		  (p.cyan() != lastpen.cyan() || 
+		   p.magenta() != lastpen.magenta() || 
+		   p.yellow() != lastpen.yellow() ||
+		   p.black() != lastpen.black()))) {
+    *out << begin << p.cyan() << " " << p.magenta() << " " << p.yellow() << " " 
+	 << p.black() << (pdf ? " k" : " setcmykcolor") << end << newl;
+  } else if(p.rgb() && (!lastpen.rgb() || 
+			(p.red() != lastpen.red() || 
+			 p.green() != lastpen.green() || 
+			 p.blue() != lastpen.blue()))) {
+    *out << begin << p.red() << " " << p.green() << " " << p.blue()
+	 << (pdf ? " rg" : " setrgbcolor") << end << newl;
+  } else if(p.grayscale() && (!lastpen.grayscale() ||
+			      p.gray() != lastpen.gray())) {
+    *out << begin << p.gray() << (pdf ? " g" : " setgray") << end << newl;
+  }
+}
+  
 void psfile::setpen(pen p)
 {
   p.convert();
   if(p == lastpen) return;
     
+  if(pdfformat) {
+    if(p.blend() != lastpen.blend()) 
+      *out << "/" << p.blend() << " .setblendmode" << newl;
+  
+    if(p.opacity() != lastpen.opacity()) 
+      *out << p.opacity() << " .setopacityalpha" << newl;
+  }
+  
   if(!p.fillpattern().empty() && p.fillpattern() != lastpen.fillpattern()) 
     *out << p.fillpattern() << " setpattern" << newl;
-  else if(p.cmyk() && (!lastpen.cmyk() ||
-		       (p.cyan() != lastpen.cyan() || 
-			p.magenta() != lastpen.magenta() || 
-			p.yellow() != lastpen.yellow() ||
-			p.black() != lastpen.black()))) {
-    *out << p.cyan() << " " << p.magenta() << " " << p.yellow() << " " 
-	 << p.black() << " setcmykcolor" << newl;
-  } else if(p.rgb() && (!lastpen.rgb() || 
-			(p.red() != lastpen.red() || 
-			 p.green() != lastpen.green() || 
-			 p.blue() != lastpen.blue()))) {
-    *out << p.red() << " " << p.green() << " " << p.blue()
-	 << " setrgbcolor" << newl;
-  } else if(p.grayscale() && (!lastpen.grayscale() ||
-			      p.gray() != lastpen.gray())) {
-    *out << p.gray() << " setgray" << newl;
-  }
-    
+  else setcolor(p);
+  
   if(p.width() != lastpen.width()) {
     *out << " 0 " << p.width() << 
       " dtransform truncate idtransform setlinewidth pop" << newl;
@@ -105,7 +148,8 @@ void psfile::setpen(pen p)
   }
     
   if(p.stroke() != lastpen.stroke()) {
-    *out << "[" << p.stroke() << "] 0 setdash" << newl;
+    *out << "[" << p.stroke() << "] " << std::setprecision(6) 
+	 << p.linetype().offset << " setdash" << newl;
   }
     
   lastpen=p;
@@ -131,8 +175,8 @@ void psfile::write(path p, bool newPath)
 
   if (n == 1) {
     moveto(p.point(0));
-    rlineto(pair(0,0));
-    stroke();
+    lineto(p.point(0));
+    return;
   }
 
   // Draw points
@@ -159,13 +203,16 @@ void psfile::shade(array *a, const bbox& b)
   size_t m=a0->size();
 
   pen *p=read<pen *>(a0,0);
+  p->convert();
   ColorSpace colorspace=p->colorspace();
+  checkColorSpace(colorspace);
+  
   unsigned ncomponents=ColorComponents[colorspace];
   
   *out << "<< /ShadingType 1" << newl
        << "/Matrix ";
 
-  write(shift(Shift)*matrix(b.Min(),b.Max()));
+  write(matrix(b.Min(),b.Max()));
   *out << newl;
   *out << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
        << "/Function" << newl
@@ -189,6 +236,7 @@ void psfile::shade(array *a, const bbox& b)
     if(aisize != m) reportError("shading matrix must be rectangular");
     for(size_t j=0; j < m; j++) {
 	pen *p=read<pen *>(ai,j);
+	p->convert();
 	if(p->colorspace() != colorspace)
 	  reportError("inconsistent shading colorspaces");
 	writeHex(p,ncomponents);
@@ -201,12 +249,14 @@ void psfile::shade(array *a, const bbox& b)
 }
 
 // Axial and radial shading
-void psfile::shade(bool axial, const string& colorspace,
+void psfile::shade(bool axial, const ColorSpace &colorspace,
 		   const pen& pena, const pair& a, double ra,
 		   const pen& penb, const pair& b, double rb)
 {
+  checkColorSpace(colorspace);
+  
   *out << "<< /ShadingType " << (axial ? "2" : "3") << newl
-       << "/ColorSpace /Device" << colorspace << newl
+       << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
        << "/Coords [";
   write(a);
   if(!axial) write(ra);
@@ -236,7 +286,9 @@ void psfile::shade(array *pens, array *vertices, array *edges)
   if(size == 0) return;
   
   pen *p=read<pen *>(pens,0);
+  p->convert();
   ColorSpace colorspace=p->colorspace();
+  checkColorSpace(colorspace);
 
   *out << "<< /ShadingType 4" << newl
        << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
@@ -245,6 +297,7 @@ void psfile::shade(array *pens, array *vertices, array *edges)
     write(read<int>(edges,i));
     write(read<pair>(vertices,i));
     pen *p=read<pen *>(pens,i);
+    p->convert();
     if(p->colorspace() != colorspace)
       reportError("inconsistent shading colorspaces");
     *out << " ";
@@ -290,6 +343,30 @@ void psfile::writeHex(pen *p, int ncomponents) {
   }
 }
 
+void psfile::imageheader(double width, double height, ColorSpace colorspace)
+{
+  unsigned ncomponents=ColorComponents[colorspace];
+  *out << "/Device" << ColorDeviceSuffix[colorspace] << " setcolorspace" 
+       << newl
+       << "<<" << newl
+       << "/ImageType 1" << newl
+       << "/Width " << width << newl
+       << "/Height " << height << newl
+       << "/BitsPerComponent 8" << newl
+       << "/Decode [";
+  
+  for(unsigned i=0; i < ncomponents; ++i)
+    *out << "0 1 ";
+  
+  *out << "]" << newl
+       << "/ImageMatrix [" << width << " 0 0 " << height << " 0 0]" << newl
+       << "/DataSource currentfile /ASCIIHexDecode filter" << newl
+       << ">>" << newl
+       << "image" << newl;
+}
+
+static const char *inconsistent="inconsistent colorspaces in palette";
+
 void psfile::image(array *a, array *P)
 {
   size_t asize=a->size();
@@ -301,27 +378,13 @@ void psfile::image(array *a, array *P)
   size_t a0size=a0->size();
   
   pen *p=read<pen *>(P,0);
+  p->convert();
   ColorSpace colorspace=p->colorspace();
+  checkColorSpace(colorspace);
   unsigned ncomponents=ColorComponents[colorspace];
   
-  *out << "/Device" << ColorDeviceSuffix[colorspace] << " setcolorspace" 
-       << newl
-       << "<<" << newl
-       << "/ImageType 1" << newl
-       << "/Width " << a0size << newl
-       << "/Height " << asize << newl
-       << "/BitsPerComponent 8" << newl
-       << "/Decode [";
-  
-  for(unsigned i=0; i < ncomponents; ++i)
-    *out << "0 1 ";
-  
-  *out << "]" << newl
-       << "/ImageMatrix [" << a0size << " 0 0 " << asize << " 0 0]" << newl
-       << "/DataSource currentfile /ASCIIHexDecode filter" << newl
-       << ">>" << newl
-       << "image" << newl;
-  
+  imageheader(a0size,asize,colorspace);
+    
   double min=read<double>(a0,0);
   double max=min;
   for(size_t i=0; i < asize; i++) {
@@ -340,10 +403,11 @@ void psfile::image(array *a, array *P)
     for(size_t j=0; j < a0size; j++) {
       double val=read<double>(ai,j);
       pen *p=read<pen *>(P,(int) ((val-min)*step+0.5));
+      p->convert();
       
       if(p->colorspace() != colorspace)
-	reportError("inconsistent colorspaces in palette");
-  
+	reportError(inconsistent);
+
       writeHex(p,ncomponents);
     }
   }
@@ -351,4 +415,37 @@ void psfile::image(array *a, array *P)
   *out << ">" << endl;
 }
 
+void psfile::image(array *a)
+{
+  size_t asize=a->size();
+  
+  if(asize == 0) return;
+  
+  array *a0=read<array *>(a,0);
+  size_t a0size=a0->size();
+  
+  pen *p=read<pen *>(a0,0);
+  p->convert();
+  ColorSpace colorspace=p->colorspace();
+  checkColorSpace(colorspace);
+  unsigned ncomponents=ColorComponents[colorspace];
+  
+  imageheader(a0size,asize,colorspace);
+    
+  for(size_t i=0; i < asize; i++) {
+    array *ai=read<array *>(a,i);
+    for(size_t j=0; j < a0size; j++) {
+      pen *p=read<pen *>(ai,j);
+      p->convert();
+
+      if(p->colorspace() != colorspace)
+	reportError(inconsistent);
+
+      writeHex(p,ncomponents);
+    }
+  }
+  
+  *out << ">" << endl;
+}
+  
 } //namespace camp
