@@ -12,6 +12,7 @@
 #include <fstream>
 #include <stack>
 #include <iomanip>
+#include <sstream>
 
 #include "pair.h"
 #include "path.h"
@@ -23,51 +24,57 @@ namespace camp {
 
 inline void BoundingBox(std::ostream& s, const bbox& box) 
 {
-  s << "%%BoundingBox: " << box.LowRes() << newl;
+  s << "%%BoundingBox: " << std::setprecision(0) << std::fixed 
+       << box.LowRes() << newl;
+  s.unsetf(std::ios::fixed);
   s << "%%HiResBoundingBox: " << std::setprecision(9) << box << newl;
 }
 
 class psfile {
   string filename;
-  bbox box;
-  pair Shift;
-  bool rawmode;
+  bool pdfformat; // Is final output format PDF?
+  bool pdf;       // Output direct PDF?
   pen lastpen;
-  ostream *out;
   std::stack<pen> pens;
 
-  void writeUnshifted(pair z) {
-    *out << " " << z.getx() << " " << z.gety();
-  }
-
   void write(transform t) {
-    *out << "[" << " " << t.getxx() << " " << t.getyx()
+    if(!pdf) *out << "[";
+    *out << " " << t.getxx() << " " << t.getyx()
 	 << " " << t.getxy() << " " << t.getyy()
-	 << " " << t.getx() << " " << t.gety() << "]";
+	 << " " << t.getx() << " " << t.gety();
+    if(!pdf) *out << "]";
   }
 
   void writeHex(unsigned int n) {
     *out << std::hex << std::setw(2) << std::setfill('0') << n << std::dec;
   }
   
+protected:
+  std::ostream *out;
+  
 public: 
-  psfile(const string& filename, const bbox& box, const pair& Shift);
+  psfile(const string& filename, bool pdfformat);
+  psfile() {pdf=settings::pdf(settings::getSetting<mem::string>("tex"));}
+
   ~psfile();
   
-  void prologue();
+  void BoundingBox(const bbox& box) {
+    camp::BoundingBox(*out,box);
+  }
+  
+  void prologue(const bbox& box);
   void epilogue();
 
-  void raw(bool mode) {rawmode=mode;}
-  bool raw() {return rawmode;}
+  void close();
   
-  void write(pair z) {
-    writeUnshifted(rawmode ? z : z+Shift);
-  }
-
   void write(double x) {
     *out << " " << x;
   }
 
+  void write(pair z) {
+    *out << " " << z.getx() << " " << z.gety();
+  }
+
   void writeHex(pen *p, int ncomponents);
   
   void resetpen() {
@@ -75,88 +82,108 @@ public:
     lastpen.convert();
   }
   
-  void setpen(pen p);
+  void setcolor(const pen& p, const string& begin, const string& end);
 
+  void setpen(pen p);
+  
   void write(pen p);
   
   void write(path p, bool newPath=true);
   
   void newpath() {
-      *out << "newpath";
+    if(!pdf) *out << "newpath";
   }
 
   void moveto(pair z) {
-      write(z);
-      *out << " moveto" << newl;
+    write(z);
+    if(pdf) *out << " m" << newl;
+    else *out << " moveto" << newl;
   }
 
   void lineto(pair z) {
-      write(z);
-      *out << " lineto" << newl;
+    write(z);
+    if(pdf) *out << " l" << newl;
+    else *out << " lineto" << newl;
   }
 
   void curveto(pair zp, pair zm, pair z1) {
-      write(zp); write(zm); write(z1);
-      *out << " curveto" << newl;
+    write(zp); write(zm); write(z1);
+    if(pdf) *out << " c" << newl;
+    else *out << " curveto" << newl;
   }
 
   void closepath() {
-      *out << " closepath" << newl;
-  }
-
-  void rlineto(pair z) {
-      writeUnshifted(z);
-      *out << " rlineto" << newl;
+    if(pdf) *out << " h" << newl;
+    else *out << " closepath" << newl;
   }
 
   void stroke() {
-    *out << " stroke" << newl;
+    if(pdf) *out << " S" << newl;
+    else *out << " stroke" << newl;
   }
   
   void fill(const pen &p) {
-    *out << (p.evenodd() ? " eofill" : " fill") << newl;
+    if(p.evenodd()) {
+      if(pdf) *out << " f*" << newl;
+      else *out << "eofill" << newl;
+    } else {
+      if(pdf) *out << " f" << newl;
+      else *out << "fill" << newl;
+    }
   }
   
   void clip(const pen &p) {
-    *out << (p.evenodd() ? " eoclip" : " clip") << newl;
+    if(p.evenodd()) {
+      if(pdf) *out << " W* n" << newl;
+      else *out << " eoclip" << newl;
+    } else {
+      if(pdf) *out << " W n" << newl;
+      else *out << " clip" << newl;
+    }
   }
   
   void shade(vm::array *a, const bbox& b);
   
-  void shade(bool axial, const string& colorspace,
+  void shade(bool axial, const ColorSpace& colorspace,
 	     const pen& pena, const pair& a, double ra,
 	     const pen& penb, const pair& b, double rb);
   
   void shade(vm::array *pens, vm::array *vertices, vm::array *edges);
   
+  void imageheader(double width, double height, ColorSpace colorspace);
+  
   void image(vm::array *a, vm::array *p);
+  void image(vm::array *a);
 
-  void gsave() {
-    *out << "gsave" << newl;
+  void gsave(bool tex=false) {
+    if(pdf) *out << "q";
+    else *out << "gsave";
+    if(!tex) *out << newl;
     pens.push(lastpen);
   }
   
-  void grestore() {
+  void grestore(bool tex=false) {
     if(pens.size() < 1)
       reportError("grestore without matching gsave");
     lastpen = pens.top();
     pens.pop();
-    *out << "grestore" << newl;
+    if(pdf) *out << "Q";
+    else *out << "grestore";
+    if(!tex) *out << newl;
   }
 
   void translate(pair z) {
-    writeUnshifted(z);
+   if(pdf) *out << " 1 0 0 1 " << newl;
+   write(z);
+   if(pdf) *out << " cm" << newl;
     *out << " translate" << newl;
   }
 
-  void concatUnshifted(transform t) {
-    write(t);
-    *out << " concat" << newl;
-  }
-  
   // Multiply on a transform to the transformation matrix.
   void concat(transform t) {
-    concatUnshifted(rawmode ? t : shift(Shift)*t);
+   write(t);
+   if(pdf) *out << " cm" << newl;
+   else *out << " concat" << newl;
   }
   
   void verbatimline(const string& s) {
