@@ -45,10 +45,27 @@ trans::access *record::initializer() {
 dummyRecord::dummyRecord(symbol *name) 
   : record(name, new frame(0,0))
 {
-  // Encode the instructions to put an placeholder instance of the record on the
-  // stack.
+  // Encode the instructions to put an placeholder instance of the record
+  // on the stack.
   trans::coder c(this, 0);
   c.closeRecord();
 }
 
+dummyRecord::dummyRecord(string s)
+  : record (symbol::trans(s), new frame(0,0))
+{
+  // Encode the instructions to put an placeholder instance of the record
+  // on the stack.
+  trans::coder c(this, 0);
+  c.closeRecord();
+}
+
+void dummyRecord::add(string name, ty *t, trans::access *a) {
+  e.addVar(symbol::trans(name), new trans::varEntry(t, a, this)); 
+}
+
+void dummyRecord::add(string name, function *t, vm::bltin f) {
+  add(name, t, new trans::bltinAccess(f));
+}
+
 } // namespace types
