@@ -11,6 +11,8 @@
 #include <cerrno>
 #include <vector>
 #include <sys/stat.h>
+#include <cfloat>
+#include <locale.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -34,6 +36,17 @@
 #include "item.h"
 #include "refaccess.h"
 
+#ifdef HAVE_LIBCURSES
+#include <curses.h>
+
+// Workaround missing symbolic link under CYGWIN
+#if defined(__CYGWIN__) && defined(HAVE_LIBNCURSES)
+#include <ncurses/term.h>
+#else
+#include <term.h>
+#endif
+#endif
+
 using std::vector;
 using vm::item;
 
@@ -41,9 +54,7 @@ using trans::itemRefAccess;
 using trans::refAccess;
 using trans::varEntry;
   
-namespace loop {
-  void doConfig(string filename);
-}
+void doConfig(string filename);
 
 namespace settings {
   
@@ -58,7 +69,7 @@ const string defaultPSViewer=
 const string defaultPDFViewer=
   "c:\\Program Files\\Adobe\\Acrobat 7.0\\Reader\\AcroRd32.exe";
 const string defaultGhostscript=
-  "c:\\Program Files\\gs\\gs8.53\\bin\\gswin32.exe";
+  "c:\\Program Files\\gs\\gs8.54\\bin\\gswin32c.exe";
 const string defaultPython="c:\\Python24\\python.exe";
 const string defaultDisplay="imdisplay";
 #undef ASYMPTOTE_SYSDIR
@@ -69,7 +80,11 @@ const bool msdos=false;
 const char *HOME="HOME";
 const char pathSeparator=':';
 const string defaultPSViewer="gv";
+#ifdef __APPLE__
+const string defaultPDFViewer="open";
+#else  
 const string defaultPDFViewer="acroread";
+#endif  
 const string defaultGhostscript="gs";
 const string defaultDisplay="display";
 const string defaultPython="";
@@ -87,8 +102,10 @@ char *argv0;
 int verbose;
 int safe=1;
   
+// Work around backwards-incompatible command-line options of gv-3.6.1.
+string gvOptionPrefix="-";
+  
 int ShipoutNumber=0;
-int scrollLines=0;
   
 const string suffix="asy";
 const string guisuffix="gui";
@@ -124,13 +141,15 @@ struct option : public gc {
   char code;      // Command line option, i.e. 'V' for -V.
   bool argument;  // If it takes an argument on the command line.  This is set
                   // based on whether argname is empty.
-  
   mem::string argname; // The argument name for printing the description.
   mem::string desc; // One line description of what the option does.
+  bool cmdlineonly; // If it is only available on the command line.
+  mem::string Default; // A string containing an optional default value.
 
-  option(mem::string name, char code, mem::string argname, mem::string desc)
-    : name(name), code(code), argument(!argname.empty()),
-      argname(argname), desc(desc) {}
+  option(mem::string name, char code, mem::string argname, mem::string desc,
+	 bool cmdlineonly=false, mem::string Default="")
+    : name(name), code(code), argument(!argname.empty()), argname(argname),
+      desc(desc), cmdlineonly(cmdlineonly), Default(Default) {}
 
   virtual ~option() {}
 
@@ -193,7 +212,10 @@ struct option : public gc {
         cerr << endl;
         cerr << std::left << std::setw(WIDTH) << "";
       }
-      cerr << desc << endl;
+      cerr << desc;
+      if(cmdlineonly) cerr << "; command-line only";
+      if(Default != "") cerr << " [" << Default << "]";
+      cerr << endl;
     }
   }
 };
@@ -204,8 +226,8 @@ struct setting : public option {
   types::ty *t;
 
   setting(mem::string name, char code, mem::string argname, mem::string desc,
-          types::ty *t)
-      : option(name, code, argname, desc), t(t) {}
+          types::ty *t, mem::string Default)
+    : option(name, code, argname, desc, false,Default), t(t) {}
 
   virtual void reset() = 0;
 
@@ -215,12 +237,17 @@ struct setting : public option {
   virtual void add() {
     option::add();
 
+#if 0
     settingsModule->e.addVar(symbol::trans(name), buildVarEntry());
+#endif
+    settingsModule->add(name, t, buildAccess());
   }
 
+#if 0
   varEntry *buildVarEntry() {
-    return new varEntry(t, buildAccess());
+    return new varEntry(t, buildAccess(), getSettingsModule());
   }
+#endif
 };
 
 struct itemSetting : public setting {
@@ -229,10 +256,9 @@ struct itemSetting : public setting {
 
   itemSetting(mem::string name, char code,
               mem::string argname, mem::string desc,
-              types::ty *t, item defaultValue)
-      : setting(name, code, argname, desc, t), defaultValue(defaultValue) {
-    reset();
-  }
+              types::ty *t, item defaultValue, mem::string Default="")
+    : setting(name, code, argname, desc, t, Default),
+      defaultValue(defaultValue) {reset();}
 
   void reset() {
     value=defaultValue;
@@ -245,7 +271,10 @@ struct itemSetting : public setting {
 
 item& Setting(string name) {
   itemSetting *s=dynamic_cast<itemSetting *>(optionsMap[name]);
-  assert(s);
+  if(!s) {
+    cerr << "Cannot find setting named '" << name << "'" << endl;
+    exit(-1);
+  }
   return s->value;
 }
   
@@ -253,7 +282,8 @@ struct boolSetting : public itemSetting {
   boolSetting(mem::string name, char code, mem::string desc,
               bool defaultValue=false)
     : itemSetting(name, code, noarg, desc,
-                  types::primBoolean(), (item)defaultValue) {}
+                  types::primBoolean(), (item)defaultValue,
+		  defaultValue ? "true" : "false") {}
 
   bool getOption() {
     value=(item)true;
@@ -290,7 +320,7 @@ struct boolSetting : public itemSetting {
     typedef mem::list<boolSetting *> setlist;
     setlist set;
     multiOption(mem::string name, char code, mem::string desc)
-      : option(name, code, noarg, desc) {}
+      : option(name, code, noarg, desc, true) {}
 
     void add(boolSetting *s) {
       set.push_back(s);
@@ -360,6 +390,21 @@ struct stringSetting : public argumentSetting {
   }
 };
 
+struct userSetting : public argumentSetting {
+  userSetting(mem::string name, char code,
+	      mem::string argname, mem::string desc,
+	      mem::string defaultValue)
+    : argumentSetting(name, code, argname, desc,
+		      types::primString(), (item)defaultValue) {}
+
+  bool getOption() {
+    mem::string s=vm::get<mem::string>(value)+mem::string(optarg);
+    s.push_back(';');
+    value=(item) s;
+    return true;
+  }
+};
+
 mem::string GetEnv(mem::string s, mem::string Default) {
   transform(s.begin(), s.end(), s.begin(), toupper);        
   string t=Getenv(("ASYMPTOTE_"+s).c_str());
@@ -384,7 +429,7 @@ struct dataSetting : public argumentSetting {
     try {
       value=(item)lexical::cast<T>(optarg);
     } catch (lexical::bad_cast&) {
-      error("option requires" + text + " as an argument");
+      error("option requires " + text + " as an argument");
       return false;
     }
     return true;
@@ -430,7 +475,6 @@ struct alignSetting : public argumentSetting {
       value=(int)BOTTOM;
     else if (str=="Z") {
       value=(int)ZERO;
-      Setting("tex")=false;
     }
     else {
       error("invalid argument for option");
@@ -447,8 +491,8 @@ struct refSetting : public setting {
 
   refSetting(mem::string name, char code, mem::string argname,
              mem::string desc, types::ty *t, T *ref, T defaultValue)
-      : setting(name, code, argname, desc, t),
-        ref(ref), defaultValue(defaultValue) {
+    : setting(name, code, argname, desc, t, "0"),
+      ref(ref), defaultValue(defaultValue) {
     reset();
   }
 
@@ -523,7 +567,8 @@ void reportSyntax() {
 void displayOptions()
 {
   cerr << endl;
-  cerr << "Options: " << endl;
+  cerr << "Options (negate by replacing - with -no): " 
+       << endl << endl;
   for (optionsMap_t::iterator opt=optionsMap.begin();
        opt!=optionsMap.end();
        ++opt)
@@ -532,7 +577,7 @@ void displayOptions()
 
 struct helpOption : public option {
   helpOption(mem::string name, char code, mem::string desc)
-    : option(name, code, noarg, desc) {}
+    : option(name, code, noarg, desc, true) {}
 
   bool getOption() {
     usage(argv0);
@@ -550,7 +595,8 @@ struct safeOption : public option {
   bool value;
 
   safeOption(mem::string name, char code, mem::string desc, bool value)
-    : option(name, code, noarg, desc), value(value) {}
+    : option(name, code, noarg, desc, true, value ? "true" : "false"),
+      value(value) {}
 
   bool getOption() {
     safe=value;
@@ -571,7 +617,7 @@ c_option *build_longopts() {
   size_t n=optionsMap.size();
 
 #ifdef USEGC
-  c_option *longopts=new (GC) c_option[n];
+  c_option *longopts=new (UseGC) c_option[n];
 #else
   c_option *longopts=new c_option[n];
 #endif
@@ -628,15 +674,21 @@ void getOptions(int argc, char *argv[])
     reportSyntax();
 }
 
+#ifdef USEGC
+void no_GCwarn(char *, GC_word) {}
+#endif
+
 void initSettings() {
   settingsModule=new types::dummyRecord(symbol::trans("settings"));
   
-  multiOption *view=new multiOption("View", 'V', "View output files");
-  view->add(new boolSetting("batchView", 0,
-			    "View output files in batch mode", false));
+  multiOption *view=new multiOption("View", 'V', "View output");
+  view->add(new boolSetting("batchView", 0, "View output in batch mode",
+			    msdos));
+  view->add(new boolSetting("multipleView", 0,
+			    "View output from multiple batch-mode files",
+			    false));
   view->add(new boolSetting("interactiveView", 0,
-			    "View output files in interactive mode", true));
-  view->add(new boolSetting("oneFileView", 0, "", msdos));
+			    "View output in interactive mode", true));
   addOption(view);
 
   addOption(new realSetting("deconstruct", 'x', "X",
@@ -645,26 +697,32 @@ void initSettings() {
   addOption(new boolSetting("clearGUI", 'c', "Clear GUI operations"));
   addOption(new boolSetting("ignoreGUI", 'i', "Ignore GUI operations"));
   addOption(new stringSetting("outformat", 'f', "format",
-			      "Convert each output file to specified format",
-			      "eps"));
+		      "Convert each output file to specified format", ""));
   addOption(new stringSetting("outname", 'o', "name",
-			      "(First) output file name", ""));
+			      "Alternative output name for first file",
+			      ""));
   addOption(new helpOption("help", 'h', "Show summary of options"));
 
-  addOption(new pairSetting("offset", 'O', "pair", "PostScript offset"));
+  addOption(new pairSetting("offset", 'O', "pair",
+			    "PostScript offset [(0,0)]"));
   addOption(new alignSetting("align", 'a', "C|B|T|Z",
-		"Center, Bottom, Top, or Zero page alignment; Z => -notex"));
+		"Center, Bottom, Top, or Zero page alignment [Center]"));
   
   addOption(new boolSetting("debug", 'd', "Enable debugging messages"));
   addOption(new incrementSetting("verbose", 'v',
 				 "Increase verbosity level", &verbose));
   addOption(new boolSetting("keep", 'k', "Keep intermediate files"));
-  addOption(new boolSetting("tex", 0,
-			    "Enable LaTeX label postprocessing (default)",
-			    true));
+  addOption(new stringSetting("tex", 0,"engine",
+			      "TeX engine (\"latex|pdflatex|tex|pdftex|none\") [\"latex\"]",
+			      "latex"));
+  addOption(new boolSetting("twice", 0,
+			    "Run LaTeX twice (to resolve references)"));
   addOption(new boolSetting("inlinetex", 0, "Generate inline tex code"));
-  addOption(new boolSetting("parseonly", 'p', "Parse test"));
-  addOption(new boolSetting("translate", 's', "Translate test"));
+  addOption(new boolSetting("parseonly", 'p', "Parse file"));
+  addOption(new boolSetting("translate", 's',
+			    "Show translated virtual machine code"));
+  addOption(new boolSetting("tabcompletion", 0,
+                            "Interactive prompt auto-completion", true));
   addOption(new boolSetting("listvariables", 'l',
 			    "List available global functions and variables"));
   
@@ -683,23 +741,36 @@ void initSettings() {
   addOption(new boolSetting("cmyk", 0, "Convert rgb colors to cmyk"));
 
   addOption(new safeOption("safe", 0,
-			   "Disable system call (default)", true));
+			   "Disable system call", true));
   addOption(new safeOption("unsafe", 0,
 			   "Enable system call", false));
-
-  addOption(new boolSetting("localhistory", 0, 
-			    "Use a local interactive history file"));
-  addOption(new intSetting("historylines", 0, "n", 
-			   "Retain n lines of history (default 1000)",1000));
-  addOption(new boolSetting("autoplain", 0,
-			    "Enable automatic importing of plain (default)",
-			    true));
   
-  addOption(new envSetting("config","config.asy"));
+  addOption(new stringSetting("prompt", 0,"string","Prompt [\"> \"]","> "));
+  addOption(new boolSetting("quiet", 'q',
+			    "Suppress welcome message"));
+  addOption(new boolSetting("localhistory", 0,
+			    "Use a local interactive history file"));
+  addOption(new intSetting("historylines", 0, "n",
+			   "Retain n lines of history [1000]",1000));
+  addOption(new intSetting("scroll", 0, "n",
+			   "Scroll standard output n lines at a time [0]",0));
+  addOption(new boolSetting("autoplain", 0,
+			    "Enable automatic importing of plain",
+			    true));
+  addOption(new boolSetting("autorotate", 0,
+			    "Enable automatic PDF page rotation",
+			    false));
+  addOption(new userSetting("user", 'u', "x",
+			    "General purpose user string  [\"\"]", ""));
+  
+  addOption(new realSetting("paperwidth", 0, "bp", ""));
+  addOption(new realSetting("paperheight", 0, "bp", ""));
+  
+  addOption(new envSetting("config","config."+suffix));
   addOption(new envSetting("pdfviewer", defaultPDFViewer));
   addOption(new envSetting("psviewer", defaultPSViewer));
   addOption(new envSetting("gs", defaultGhostscript));
-  addOption(new envSetting("latex", "latex"));
+  addOption(new envSetting("texpath", ""));
   addOption(new envSetting("dvips", "dvips"));
   addOption(new envSetting("convert", "convert"));
   addOption(new envSetting("display", defaultDisplay));
@@ -715,9 +786,16 @@ int numArgs() { return argCount; }
 char *getArg(int n) { return argList[n]; }
 
 void setInteractive() {
-  if(numArgs() == 0 && !getSetting<bool>("listvariables"))
+  if(numArgs() == 0 && !getSetting<bool>("listvariables")) {
     interact::interactive=true;
-  historyname=getSetting<bool>("localhistory") ? ".asy_history" 
+    
+    // Work around backwards-incompatible command-line options of gv-3.6.1.
+    if(!msdos && (getSetting<mem::string>("pdfviewer") == "gv" ||
+		  getSetting<mem::string>("psviewer") == "gv"))
+      gvOptionPrefix=System("gv --version",2) == 0 ? "--" : "-";
+  }
+  
+  historyname=getSetting<bool>("localhistory") ? "."+suffix+"_history" 
     : (initdir+"/history");
 }
 
@@ -725,8 +803,8 @@ bool view() {
   if (interact::interactive)
     return getSetting<bool>("interactiveView");
   else
-    return getSetting<bool>("batchView") ||
-           (numArgs()==1 && getSetting<bool>("oneFileView"));
+    return getSetting<bool>("batchView") && 
+      (numArgs() == 1 || getSetting<bool>("multipleView"));
 }
 
 bool trap() {
@@ -736,8 +814,13 @@ bool trap() {
     return !getSetting<bool>("batchMask");
 }
 
+mem::string outname() {
+  mem::string name=getSetting<mem::string>("outname");
+  return name.empty() ? "out" : name;
+}
+
 void initDir() {
-  initdir=Getenv(HOME,false)+"/.asy";
+  initdir=Getenv(HOME,false)+"/."+suffix;
   mkdir(initdir.c_str(),0xFFFF);
 }
   
@@ -759,15 +842,23 @@ void setPath() {
 #endif
 }
 
-void GetPageDimensions(double& pageWidth, double& pageHeight) {
+void SetPageDimensions() {
   string paperType=getSetting<mem::string>("papertype");
 
+  if(paperType == "" &&
+     getSetting<double>("paperwidth") != 0.0 &&
+     getSetting<double>("paperheight") != 0.0) return;
+  
+  const double inches=72;
+  const double cm=inches/2.540005;
+  
   if(paperType == "letter") {
-    pageWidth=72.0*8.5;
-    pageHeight=72.0*11.0;
+    Setting("paperwidth")=8.5*inches;
+    Setting("paperheight")=11.0*inches;
   } else {
-    pageWidth=72.0*21.0/2.54;
-    pageHeight=72.0*29.7/2.54;
+    Setting("paperwidth")=21.0*cm;
+    Setting("paperheight")=29.7*cm;
+    
     if(paperType != "a4") {
       cerr << "Unknown paper size \'" << paperType << "\'; assuming a4." 
 	   << endl;
@@ -776,10 +867,77 @@ void GetPageDimensions(double& pageWidth, double& pageHeight) {
   }
 }
 
+bool pdf(const mem::string& texengine) {
+  return texengine == "pdflatex" || texengine == "pdftex";
+}
+
+bool latex(const mem::string& texengine) {
+  return texengine == "latex" || texengine == "pdflatex";
+}
+
+// TeX special command to set up currentmatrix for typesetting labels.
+const char *beginlabel(const mem::string& texengine) {
+  if(pdf(texengine))
+    return "\\special{pdf: q #5 0 0 cm}\\wd\\ASYbox 0pt\\dp\\ASYbox 0pt\\ht\\ASYbox 0pt";
+      return "\\special{ps: gsave currentpoint currentpoint translate [#5 0 0] concat neg exch neg exch translate}";
+}
+
+// TeX special command to restore currentmatrix after typesetting labels.
+const char *endlabel(const mem::string& texengine) {
+  if(pdf(texengine))
+    return "\\special{pdf: Q}";
+  return "\\special{ps: currentpoint grestore moveto}";
+}
+
+// TeX special command to clip a label
+const char *clip(const mem::string& texengine) {
+  if(pdf(texengine))
+    return "\\special{pdf: #3}";
+  return "\\special{ps: currentpoint currentpoint translate matrix currentmatrix\n[matrix defaultmatrix 0 get 0 0 matrix defaultmatrix 3 get\nmatrix currentmatrix 4 get matrix currentmatrix 5 get] setmatrix\n#3\nsetmatrix neg exch neg exch translate}";
+}
+
+// Begin TeX special command.
+const char *beginspecial(const mem::string& texengine) {
+  if(pdf(texengine))
+    return "\\special{pdf: ";
+  return "\\special{ps: ";
+}
+
+// End TeX special command.
+const char *endspecial() {
+  return "}";
+}
+
+mem::string texengine() {
+  mem::string path=getSetting<mem::string>("texpath");
+  mem::string tex=getSetting<mem::string>("tex");
+  return (path == "")  ? tex : path+"/"+tex;
+}
+
+int getScroll() 
+{
+  int scroll=settings::getSetting<int>("scroll");
+#ifdef HAVE_LIBCURSES  
+  if(scroll < 0) {
+    char *terminal=getenv("TERM");
+    if(terminal) {
+      setupterm(terminal,1,NULL);
+      scroll=lines > 2 ? lines-1 : 1;
+    }
+  }
+#endif
+  return scroll;
+}
+
 void setOptions(int argc, char *argv[])
 {
   argv0=argv[0];
 
+  if(setlocale (LC_ALL, "") == NULL)
+    perror("setlocale");
+  
+  cout.precision(DBL_DIG);
+  
   // Make configuration and history directory
   initDir();
   
@@ -792,7 +950,7 @@ void setOptions(int argc, char *argv[])
   
   // Read user configuration file.
   setPath();
-  loop::doConfig(getSetting<mem::string>("config"));
+  doConfig(getSetting<mem::string>("config"));
   
   // Remember any changes to the defaultpen.
   initialdefaultpen=new camp::pen(defaultpen);
@@ -800,6 +958,10 @@ void setOptions(int argc, char *argv[])
   // Read command-line options again to override configuration file defaults.
   getOptions(argc,argv);
   
+#ifdef USEGC
+  if(!getSetting<bool>("debug")) GC_set_warn_proc(no_GCwarn);
+#endif  
+
   // Set variables for the normal arguments.
   argCount = argc - optind;
   argList = argv + optind;
@@ -807,6 +969,12 @@ void setOptions(int argc, char *argv[])
   // Recompute search path.
   setPath();
   
+  if(getSetting<double>("paperwidth") != 0.0 && 
+     getSetting<double>("paperheight") != 0.0)
+    Setting("papertype")=mem::string("");
+  
+  SetPageDimensions();
+  
   setInteractive();
 }
 
