@@ -40,10 +40,8 @@ extern const string guisuffix;
 extern bool TeXinitialized; // Is LaTeX process initialized?
 extern mem::string historyname;
   
-void GetPageDimensions(double& pageWidth, double& pageHeight);
+void SetPageDimensions();
 
-extern int scrollLines;
-  
 types::record *getSettingsModule();
 
 vm::item& Setting(string name);
@@ -55,14 +53,29 @@ inline T getSetting(string name)
 }
 
 extern int verbose;
+extern string gvOptionPrefix;
 
 bool view();
 bool trap();
+mem::string outname();
 
 void setOptions(int argc, char *argv[]);
 
 // Access the arguments once options have been parsed.
 int numArgs();
 char *getArg(int n);
+ 
+int getScroll();
+  
+bool pdf(const mem::string& texengine);
+bool latex(const mem::string& texengine);
+  
+const char *beginlabel(const mem::string& texengine);
+const char *endlabel(const mem::string& texengine);
+const char *clip(const mem::string& texengine);
+const char *beginspecial(const mem::string& texengine);
+const char *endspecial();
+  
+mem::string texengine();
 }
 #endif
