@@ -127,61 +127,61 @@ trans::varEntry *primitiveTy::virtualField(symbol *id, signature *sig)
     case ty_pair:
       if (sig == 0 && id == symbol::trans("x")) {
         static trans::bltinAccess a(run::pairXPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("y")) {
         static trans::bltinAccess a(run::pairYPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       break;
     case ty_triple:
       if (sig == 0 && id == symbol::trans("x")) {
         static trans::bltinAccess a(run::tripleXPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("y")) {
         static trans::bltinAccess a(run::tripleYPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("z")) {
         static trans::bltinAccess a(run::tripleZPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       break;
     case ty_transform:
       if (sig == 0 && id == symbol::trans("x")) {
         static trans::bltinAccess a(run::transformXPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("y")) {
         static trans::bltinAccess a(run::transformYPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("xx")) {
         static trans::bltinAccess a(run::transformXXPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("xy")) {
         static trans::bltinAccess a(run::transformXYPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("yx")) {
         static trans::bltinAccess a(run::transformYXPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
       if (sig == 0 && id == symbol::trans("yy")) {
         static trans::bltinAccess a(run::transformYYPart);
-        static trans::varEntry v(primReal(), &a);
+        static trans::varEntry v(primReal(), &a, 0);
         return &v;
       }
     default:
@@ -245,6 +245,22 @@ ty *array::appendType()
   return appendtype;
 }
 
+ty *array::insertType()
+{
+  if (inserttype == 0)
+    inserttype = new function(celltype,formal(primInt(),"i"),
+			      formal(celltype,"x"));
+  return inserttype;
+}
+
+ty *array::deleteType()
+{
+  if (deletetype == 0)
+    deletetype = new function(celltype,formal(primInt(),"i"));
+
+  return deletetype;
+}
+
 ty *cyclicType() {
   return new function(primVoid(),formal(primBoolean(),"b"));
 }
@@ -256,6 +272,8 @@ ty *array::virtualFieldGetType(symbol *id)
     id == symbol::trans("push") ? pushType() : 
     id == symbol::trans("pop") ? popType() : 
     id == symbol::trans("append") ? appendType() : 
+    id == symbol::trans("insert") ? insertType() : 
+    id == symbol::trans("delete") ? deleteType() : 
     ty::virtualFieldGetType(id);
 }
 
@@ -264,20 +282,20 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   if (sig == 0 && id == symbol::trans("length"))
   {
     static trans::bltinAccess a(run::arrayLength);
-    static trans::varEntry v(primInt(), &a);
+    static trans::varEntry v(primInt(), &a, 0);
     return &v;
   }
   if (sig == 0 && id == symbol::trans("cyclicflag"))
   {
     static trans::bltinAccess a(run::arrayCyclicFlag);
-    static trans::varEntry v(primBoolean(), &a);
+    static trans::varEntry v(primBoolean(), &a, 0);
     return &v;
   }
   if (id == symbol::trans("cyclic") &&
       equivalent(sig, cyclicType()->getSignature()))
   {
     static trans::bltinAccess a(run::arrayCyclic);
-    static trans::varEntry v(cyclicType(), &a);
+    static trans::varEntry v(cyclicType(), &a, 0);
     return &v;
   }
   if (id == symbol::trans("push") &&
@@ -285,7 +303,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayPush);
     // v needs to be dynamic, as the push type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(pushType(), &a);
+    trans::varEntry *v = new trans::varEntry(pushType(), &a, 0);
 
     return v;
   }
@@ -294,7 +312,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayPop);
     // v needs to be dynamic, as the pop type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(popType(), &a);
+    trans::varEntry *v = new trans::varEntry(popType(), &a, 0);
 
     return v;
   }
@@ -303,7 +321,25 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayAppend);
     // v needs to be dynamic, as the append type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(appendType(), &a);
+    trans::varEntry *v = new trans::varEntry(appendType(), &a, 0);
+
+    return v;
+  }
+  if (id == symbol::trans("insert") &&
+      equivalent(sig, insertType()->getSignature()))
+  {
+    static trans::bltinAccess a(run::arrayInsert);
+    // v needs to be dynamic, as the insert type differs among arrays.
+    trans::varEntry *v = new trans::varEntry(insertType(), &a, 0);
+
+    return v;
+  }
+  if (id == symbol::trans("delete") &&
+      equivalent(sig, deleteType()->getSignature()))
+  {
+    static trans::bltinAccess a(run::arrayDelete);
+    // v needs to be dynamic, as the delete type differs among arrays.
+    trans::varEntry *v = new trans::varEntry(deleteType(), &a, 0);
 
     return v;
   }
