@@ -218,10 +218,12 @@ struct array : public ty {
   ty *pushtype;
   ty *poptype;
   ty *appendtype;
+  ty *inserttype;
+  ty *deletetype;
 
   array(ty *celltype)
-    : ty(ty_array), celltype(celltype), pushtype(0), poptype(0), appendtype(0)
-  {}
+    : ty(ty_array), celltype(celltype), pushtype(0), poptype(0),
+      appendtype(0), inserttype(0), deletetype(0) {}
 
   virtual bool isReference() {
     return true;
@@ -236,12 +238,21 @@ struct array : public ty {
     return 1007 * celltype->hash();
   }
 
+  int depth() {
+    if (array *cell=dynamic_cast<array *>(celltype))
+      return cell->depth() + 1;
+    else
+      return 1;
+  }
+
   void print(ostream& out) const
     { out << *celltype << "[]"; }
 
   ty *pushType();
   ty *popType();
   ty *appendType();
+  ty *insertType();
+  ty *deleteType();
 
   // Initialize to an empty array by default.
   trans::access *initializer();
@@ -390,6 +401,19 @@ struct function : public ty {
     add(f1);
     add(f2);
   }
+  function(ty *result, formal f1, formal f2, formal f3)
+    : ty(ty_function), result(result) {
+    add(f1);
+    add(f2);
+    add(f3);
+  }
+  function(ty *result, formal f1, formal f2, formal f3, formal f4)
+    : ty(ty_function), result(result) {
+    add(f1);
+    add(f2);
+    add(f3);
+    add(f4);
+  }
   virtual ~function() {}
 
   void add(formal f) {
