@@ -13,6 +13,9 @@
 #include <cerrno>
 #include <sys/wait.h>
 #include <sys/param.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
 
@@ -48,6 +51,13 @@ void backslashToSlash(string& s)
     s[p]='/';
 }
 
+void spaceToUnderscore(string& s) 
+{
+  size_t p;
+  while ((p=s.find(' ')) < string::npos)
+    s[p]='_';
+}
+
 #ifdef __CYGWIN__
 string Getenv(const char *name, bool quote)
 {
@@ -142,6 +152,12 @@ char **args(const char *command)
     }
   }
   
+  if(settings::verbose > 1) {
+    cerr << argv[0];
+    for(size_t m=1; m < n; ++m) cerr << " " << argv[m];
+    cerr << endl;
+  }
+  
   argv[n]=NULL;
   return argv;
 }
@@ -165,13 +181,14 @@ void execError(const char *command, const char *hint, const char *application)
     exit(-1);
 }
 						    
-int System(const char *command, bool quiet, bool wait,
+// quiet: 0=none; 1=suppress stdout; 2=suppress stdout+stderr.
+
+int System(const char *command, int quiet, bool wait,
 	   const char *hint, const char *application, int *ppid)
 {
   int status;
 
   if(!command) return 1;
-  if(settings::verbose > 1) cerr << command << endl;
 
   cout.flush(); // Flush stdout to avoid duplicate output.
     
@@ -183,7 +200,15 @@ int System(const char *command, bool quiet, bool wait,
   
   if(pid == 0) {
     if(interact::interactive) signal(SIGINT,SIG_IGN);
-    if(quiet) close(STDOUT_FILENO);
+    if(quiet) {
+      static int null=creat("/dev/null",O_WRONLY);
+      close(STDOUT_FILENO);
+      dup2(null,STDOUT_FILENO);
+      if(quiet == 2) {
+	close(STDERR_FILENO);
+	dup2(null,STDERR_FILENO);
+      }
+    }
     if(argv) {
       execvp(argv[0],argv);
       execError(argv[0],hint,application);
@@ -195,9 +220,11 @@ int System(const char *command, bool quiet, bool wait,
     if(waitpid(pid, &status, wait ? 0 : WNOHANG) == -1) {
       if(errno == ECHILD) return 0;
       if(errno != EINTR) {
-        ostringstream msg;
-        msg << "Command failed: " << command;
-        camp::reportError(msg);
+	if(quiet < 2) {
+	  ostringstream msg;
+	  msg << "Command failed: " << command;
+	  camp::reportError(msg);
+	}
       }
     } else {
       if(!wait) return 0;
@@ -211,15 +238,17 @@ int System(const char *command, bool quiet, bool wait,
 	}
 	return WEXITSTATUS(status);
       } else {
-        ostringstream msg;
-        msg << "Command exited abnormally: " << command;
-        camp::reportError(msg);
+	if(quiet < 2) {
+	  ostringstream msg;
+	  msg << "Command exited abnormally: " << command;
+	  camp::reportError(msg);
+	}
       }
     }
   }
 }
 
-int System(const ostringstream& command, bool quiet, bool wait,
+int System(const ostringstream& command, int quiet, bool wait,
 	   const char *hint, const char *application, int *pid)
 {
   return System(command.str().c_str(),quiet,wait,hint,application,pid);
@@ -275,3 +304,21 @@ int setPath(const char *s)
     return chdir(s);
   } return 0;
 }
+
+void popupHelp() {
+  // If the popped-up help is already running, pid stores the pid of the viewer.
+  static int pid=0;
+
+  // Status is ignored.
+  static int status=0;
+
+  // If the help viewer isn't running (or its last run has termined), launch the
+  // viewer again.
+  if (pid==0 || (waitpid(pid, &status, WNOHANG) == pid)) {
+    ostringstream cmd;
+    cmd << "'" << getSetting<mem::string>("pdfviewer") << "' " 
+      << docdir << "/asymptote.pdf";
+    status=System(cmd,0,false,"pdfviewer","your PDF viewer",&pid);
+  }
+}
+
