@@ -36,10 +36,10 @@ bool checkFormatString(const string& format);
 
 // Similar to the standard system call except allows interrupts and does
 // not invoke a shell.
-int System(const char *command, bool quiet=false, bool wait=true,
+int System(const char *command, int quiet=0, bool wait=true,
 	   const char *hint=NULL, const char *application="",
 	   int *pid=NULL);
-int System(const ostringstream& command, bool quiet=false, bool wait=true,
+int System(const ostringstream& command, int quiet=0, bool wait=true,
 	   const char *hint=NULL, const char *application="",
 	   int *pid=NULL); 
   
@@ -62,6 +62,7 @@ extern "C" double hypot(double x, double y) throw();
 extern "C" double jn(int n, double x);
 extern "C" double yn(int n, double x);
 extern "C" int fileno(FILE *);
+extern "C" char *strptime(const char *s, const char *format, struct tm *tm);
 #endif
 
 extern bool False;
@@ -76,8 +77,12 @@ char *getPath(char *p=currentpath);
 int setPath(const char *s);
 
 void backslashToSlash(string& s);
+void spaceToUnderscore(string& s);
 string Getenv(const char *name, bool quote=true);
 
 void execError(const char *command, const char *hint, const char *application);
   
+// This invokes a viewer to display the manual.  Subsequent calls will only
+// pop-up a new viewer if the old one has been closed.
+void popupHelp();
 #endif
